
#ifndef __DIBDDSURFACE_H__
#define __DIBDDSURFACE_H__

#include <windows.h>
#include <ddraw.h>
#include <deque>
#include "dib.h"

#define RGB16g(r,g,b) (((r)>>3)<<11)|(((g)>>2)<<5)|((b)>>3)
#define RGB16(r,g,b) (((r)>>3)<<10)|(((g)>>3)<<5)|((b)>>3)
#define RGB24(r,g,b) ((r)<<16)|((g)<<8)|(b)
#define RGB24to16g(c) RGB16g(((c)>>16)&0xff,((c)>>8)&0xff,(c)&0xff)
#define RGB24to16(c) RGB16(((c)>>16)&0xff,((c)>>8)&0xff,(c)&0xff)

using namespace std;

class CBaseDIB;

//class CDIB_DDSurface_Access;
class CDIB_DDSurface;
class CDDInterface;

typedef CDIB_DDSurface* lpsurface;

typedef deque<lpsurface> surfacelist;

class CDDInterface
{
public:
	CDDInterface();
	CreateFullScreen(RECT &rect,HWND hwnd,DWORD bit=32);
	CreateWindowed(RECT &rect,HWND hwnd);
	Release();
	CreateSurface(LPDDSURFACEDESC2 ddsd,CDIB_DDSurface &sf);

	DeleteSurface(CDIB_DDSurface &dsf);
	~CDDInterface();

	Flip();

	Blt(int x0,int y0,CDIB_DDSurface &dib);
	BltT(int x0,int y0,CDIB_DDSurface &dib,DWORD trans=DDBLTFAST_SRCCOLORKEY);
	BltRect(int x0,int y0,CDIB_DDSurface &dib,RECT rct,DWORD trans=DDBLTFAST_SRCCOLORKEY);
	BltRectClip(int x0,int y0,CDIB_DDSurface &dib,RECT rct);
	BltSize(RECT dest,CDIB_DDSurface &dib,DWORD trans=DDBLT_KEYSRC);
	BltSizeRect(RECT dest,CDIB_DDSurface &dib,RECT src,DWORD trans=DDBLT_KEYSRC);

	LPDIRECTDRAW7 GetDirectDraw()const;
	LPDIRECTDRAWSURFACE7 GetSurface()const;
	LPDIRECTDRAWSURFACE7 GetPrimary()const;

	HDC GetDC()const;
	ReleaseDC()const;

	bool GetDIB(CBaseDIB &dib);

	bool IsFullScreen()const;
	const RECT &GetWindowSize()const;
	Update()const;

	RestoreAllSurfaces();

private:
	AddSurface(CDIB_DDSurface &dsf);

private:

	HWND m_hwnd;
	mutable HDC m_hdc;
	mutable int m_hdccount;

	LPDIRECTDRAW7 m_lpDD;

	int m_Count;
	surfacelist m_List;

	//vC}T[tF[X
	LPDIRECTDRAWSURFACE7 m_lpDDPrimary;
	DDSURFACEDESC2 m_ddsdpr;
	//obNobt@
	LPDIRECTDRAWSURFACE7 m_lpDDBack;
	DDSURFACEDESC2 m_ddsdbk;

	bool m_IsFullScreen;
	mutable RECT m_WindowRect;
	mutable POINT m_WindowSize;

};

// Off Screen Surface
class CDIB_DDSurface
{
public:
	CDIB_DDSurface();
	Create(CDDInterface &dd,const CBaseDIB &dib);
	Create(CDDInterface &dd,RECT &rect);
	Create(CDDInterface &dd,char *filename);
	LPDIRECTDRAWSURFACE7 GetSurface();
	Restore();
	Release();
	~CDIB_DDSurface();

	Blt(int x0,int y0,CDIB_DDSurface &dib,DWORD trans=DDBLTFAST_SRCCOLORKEY);

	SetColorKey(LONG low,LONG high);
	POINT GetSize(){POINT p={m_ddsd.dwWidth,m_ddsd.dwHeight};return p;}

	friend class CDDInterface;

protected:
	DrawDIB(const CBaseDIB &dib);
	int MaskShift(DWORD mask);//}XÑVtgԂ
protected:
	LPDIRECTDRAWSURFACE7 m_lpDDsurface;
	DDSURFACEDESC2 m_ddsd;

	CBaseDIB m_lastdib;
	bool m_hasdib;


	CDDInterface *m_parent;
private:
	int m_ID;
};

/*

class CDIB_DDSurface_Access
{
public:
	CDIB_DDSurface_Access(CDIB_DDSurface &dib);
	virtual Pset(int x,int y,int color);
	virtual int Point(int x,int y)const;
protected:
	CDIB_DDSurface &m_targetDIB;
};

class CDIB_DDSurface_Access_32:public CDIB_DDSurface_Access
{
	CDIB_DDSurface_Access_32(CDIB_DDSurface &dib);
	Pset(int x,int y,int color);
	int Point(int x,int y)const;
private:
	int r;
	int g;
	int b;
};

class CDIB_DDSurface_Access_16_565:public CDIB_DDSurface_Access
{
	CDIB_DDSurface_Access_16_565(CDIB_DDSurface &dib);
	Pset(int x,int y,int color);
	int Point(int x,int y)const;
};*/

#endif

