#ifndef __MATRIX_H__
#define __MATRIX_H__

#pragma once

//for̓WJ
//#pragma warning( disable : 8027 )

#include <math.h>
#include <algorithm>
#include <ostream>
#include <vector>
#include <list>

const double PI=3.14159265358979;

#define for if(0);else for

namespace Rydot
{

template<class T> struct Vector2;
template<class T> struct Vector3;
template<class T> struct Quaternion;
template<class T> struct Matrix22;
template<class T> struct Matrix33;
template<class T> struct Matrix44;
template<class T> struct Rect2;
template<class T> struct Rect3;

typedef Vector2<float> Vector2f;
typedef Vector3<float> Vector3f;
typedef Quaternion<float> Quaternionf;
typedef Matrix22<float> Matrix22f;
typedef Matrix33<float> Matrix33f;
typedef Matrix44<float> Matrix44f;
typedef Rect2<float> Rect2f;
typedef Rect3<float> Rect3f;

typedef Vector2<int> Vector2i;
typedef Vector3<int> Vector3i;

typedef Vector2<double> Vector2d;
typedef Vector3<double> Vector3d;
typedef Quaternion<double> Quaterniond;
typedef Matrix22<double> Matrix22d;
typedef Matrix33<double> Matrix33d;
typedef Matrix44<double> Matrix44d;
typedef Rect2<double> Rect2d;
typedef Rect3<double> Rect3d;

typedef std::pair<int,int> iipair;
typedef std::vector<iipair> iipairv;

typedef std::vector<int> intv;
typedef std::vector<float> floatv;
typedef std::vector<Vector3f> Vector3fv;
typedef std::vector<Vector2f> Vector2fv;
typedef std::vector<Quaternionf> Quaternionfv;
typedef std::vector<Vector3i> Vector3iv;
typedef std::vector<Vector2i> Vector2iv;

const int sini[]={0,1,0,-1};

/////////////////////////////
// 2xNg

//܂ɕf̉ZĂ肷
template <class T>
struct Vector2
{
public:
	T x,y;
public:
	Vector2(){}
	Vector2(const T _x,const T _y):x(_x),y(_y){}
	Vector2(const Vector2 &_v):x(_v.x),y(_v.y){}
	Vector2(const T *_a):x(_a[0]),y(_a[1]){}
	//ϊ
	Vector2(const Vector3<T> &_v);
	Vector2(const Quaternion<T> &_v);

	Vector2 &operator=(const Vector2 &_v){x=_v.x;y=_v.y;return *this;}

	void ToArray(T *_a)const{_a[0]=x;_a[1]=y;}

	Vector2 &Translate(const T _x,const T _y){x+=_x;y+=_y;return *this;}
	Vector2 &Translate(const Vector2 &_v){x+=_v.x;y+=_v.y;return *this;}

	Vector2 &operator+=(const Vector2 &_v){x+=_v.x;y+=_v.y;return *this;}
	Vector2 &operator-=(const Vector2 &_v){x-=_v.x;y-=_v.y;return *this;}
	Vector2 &operator*=(const T _m){x*=_m;y*=_m;return *this;}
	//f̊|Z
	Vector2 &operator*=(const Vector2 &_v){Vector2 v(x*_v.x-y*_v.y,x*_v.y+y*_v.x);*this=v;return *this;}

	Vector2 operator+(const Vector2 &_v)const{return Vector2<T>(x+_v.x,y+_v.y);}
	Vector2 operator-(const Vector2 &_v)const{return Vector2<T>(x-_v.x,y-_v.y);}
	Vector2 operator-()const{Vector2 v(-x,-y);return v;}
	Vector2 operator*(const T _m)const{return Vector2<T>(x*_m,y*_m);}
	//f̊|Z
	Vector2 operator*(const Vector2 &_v)const{return Vector2<T>(x*_v.x-y*_v.y,x*_v.y+y*_v.x);}

	bool operator==(const Vector2 &_v)const{return (x==_v.x)&&(y==_v.y);}
	bool operator!=(const Vector2 &_v)const{return (x!=_v.x)||(y!=_v.y);}

	//Βl()
	double Abs()const{return sqrt(x*x+y*y);}
	//Βl2
	T Abs2()const{return x*x+y*y;}
	
	//
	double Dist(const Vector2 &_v)const{Vector2 v=*this-_v;return v.Abs();}
	//2
	T Dist2(const Vector2 &_v)const{Vector2 v=*this-_v;return v.Abs2();}

	//K
	//not for int
	Vector2 Norm()const{double d=sqrt(x*x+y*y);if(d==0){return Vector2<T>(1,0);}return Vector2<T>(x/d,y/d);}
	Vector2 &Normalize(){double d=sqrt(x*x+y*y);if(d==0){x=1;y=0;return *this;}x/=d;y/=d;return *this;}
	
	//f̋
	Vector2 Conj()const{return Vector2<T>(x,-y);}
	Vector2 &Conjugate(){y=-y;return *this;}

	//evfώZ(ς̃Ȃ)
	Vector2 MultEach(const Vector2 &_v)const{return Vector2<T>(x*_v.x,y*_v.y);}
	Vector2 &MultiplyEach(const Vector2 &_v){x*=_v.x;y*=_v.y;return *this;}

	//_aWA]
	//not for int
	Vector2 &Rotate(const double _a)
	{
		double s=sin(_a),c=cos(_a);
		Vector2 r(x*c-y*s,x*s+y*c);*this=r;return *this;
	}
	
	//90deg_r{]
	Vector2 &Rotate90(const int _r)
	{
		const int rr=_r&3;
		const int s=sini[_r],c=sini[(_r+1)&3];
		Vector2 r(x*c-y*s,x*s+y*c);*this=r;return *this;
	}
	
	//
	T DotProd(const Vector2 &_v)const{return x*_v.x+y*_v.y;}
	//O
	T ExProd(const Vector2 &_v)const{return x*_v.y-y*_v.x;}
	
	//x=1,y=0̕0xÃWAp
	double Angle()const{return atan2(y,x);}
	
	//xy
	void Swap(){std::swap(x,y);}
};

//intp֐

//int͉]90xɂĂقB
template <>
Vector2<int> &Vector2<int>::Rotate(const double _a)
{
	int i=((int)floor(_a*2/PI+0.5))&3;
	return Rotate90(i);
}

//o
template<class T>
std::ostream &operator<<(std::ostream &_o,const Vector2<T> &_v)
{_o<<_v.x<<","<<_v.y;return _o;}

/////////////////////////////
// 3xNg

template <class T>
struct Vector3
{
public:
	T x,y,z;
public:
	Vector3(){}
	Vector3(const T _x,const T _y,const T _z):x(_x),y(_y),z(_z){}
	Vector3(const Vector3 &_v):x(_v.x),y(_v.y),z(_v.z){}
	Vector3(const T *_a):x(_a[0]),y(_a[1]),z(_a[2]){}
	//ϊ
	Vector3(const Vector2<T> &_v);
	Vector3(const Quaternion<T> &_v);

	Vector3 &operator=(const Vector3 &_v){x=_v.x;y=_v.y;z=_v.z;return *this;}

	void ToArray(T *_a)const{_a[0]=x;_a[1]=y;_a[2]=z;}

	Vector3 &Translate(const T _x,const T _y,const T _z){x+=_x;y+=_y;z+=_z;return *this;}
	Vector3 &Translate(const Vector3 &_v){x+=_v.x;y+=_v.y;z+=_v.z;return *this;}

	Vector3 &operator+=(const Vector3 &_v){x+=_v.x;y+=_v.y;z+=_v.z;return *this;}
	Vector3 &operator-=(const Vector3 &_v){x-=_v.x;y-=_v.y;z-=_v.z;return *this;}
	Vector3 &operator*=(const T _m){x*=_m;y*=_m;z*=_m;return *this;}

	Vector3 operator+(const Vector3 &_v)const{return Vector3<T>(x+_v.x,y+_v.y,z+_v.z);}
	Vector3 operator-(const Vector3 &_v)const{return Vector3<T>(x-_v.x,y-_v.y,z-_v.z);}
	Vector3 operator-()const{return Vector3<T>(-x,-y,-z);}
	Vector3 operator*(const T _m)const{return Vector3<T>(x*_m,y*_m,z*_m);}

	bool operator==(const Vector3 &_v)const{return (x==_v.x)&&(y==_v.y)&&(z==_v.z);}
	bool operator!=(const Vector3 &_v)const{return (x!=_v.x)||(y!=_v.y)||(z!=_v.z);}

	//Βl()
	double Abs()const{return sqrt(x*x+y*y+z*z);}
	T Abs2()const{return x*x+y*y+z*z;}

	//
	double Dist(const Vector3 &_v)const{Vector3 v=*this-_v;return v.Abs();}
	T Dist2(const Vector3 &_v)const{Vector3 v=*this-_v;return v.Abs2();}

	//evfꂼώZ
	Vector3 MultEach(const Vector3 &_v)const{return Vector3<T>(x*_v.x,y*_v.y,z*_v.z);}
	Vector3 &MultiplyEach(const Vector3 &_v){x*=_v.x;y*=_v.y;z*=_v.z;return *this;}

	//K
	//not for int
	Vector3 Norm()const
	{
		const double d=sqrt(x*x+y*y+z*z);
		if(d==0)return Vector3<T>(1,0,0);
		return Vector3<T>(x/d,y/d,z/d);
	}
	Vector3 &Normalize()
	{
		const double d=sqrt(x*x+y*y+z*z);
		if(d==0){x=1;y=0;z=0;return *this;}
		x/=d;y/=d;z/=d;
		return *this;
	}

	//
	T DotProd(const Vector3 &_v)const{return x*_v.x+y*_v.y+z*_v.z;}
	//O
	Vector3 ExProd(const Vector3 &_v)const
	{
		Vector3<T> v(y*_v.z-z*_v.y,z*_v.x-x*_v.z,x*_v.y-y*_v.x);
		return v;
	}
	
	//Oς̍̃xNg̍s\
	Matrix33<T> Skew()const{return Matrix33<T>(
		 0,-z, y,
		 z, 0,-x,
		-y, x, 0
		);
	}
	
	//]
	//not for int
	Vector3 &RotateZ(const double r)
	{
		const double s=sin(r),c=cos(r);
		const double _x=c*x-s*y,_y=s*x+c*y;x=_x;y=_y;
		return *this;
	}
	Vector3 &RotateX(const double r)
	{
		const double s=sin(r),c=cos(r);
		const double _y=c*y-s*z,_z=s*y+c*z;y=_y;z=_z;
		return *this;
	}
	Vector3 &RotateY(const double r)
	{
		const double s=sin(r),c=cos(r);
		const double _z=c*z-s*x,_x=s*z+c*x;z=_z;x=_x;
		return *this;
	}
	
	Vector3 &RotateZ90(const int r)
	{
		const int i=r&3;const int s=sini[i],c=sini[(i+1)&3];
		const T _x=c*x-s*y,_y=s*x+c*y;x=_x;y=_y;
		return *this;
	}
	Vector3 &RotateX90(const int r)
	{
		const int i=r&3;const int s=sini[i],c=sini[(i+1)&3];
		const T _y=c*y-s*z,_z=s*y+c*z;y=_y;z=_z;
		return *this;
	}
	Vector3 &RotateY90(const int r)
	{
		const int i=r&3;const int s=sini[i],c=sini[(i+1)&3];
		const T _z=c*z-s*x,_x=s*z+c*x;z=_z;x=_x;
		return *this;
	}
};

//Ƃ肠intp֐

//90x܂킵ĂB
template<> Vector3<int> &Vector3<int>::RotateZ(const double r)
{const int i=((int)floor(r*2/PI+0.5))&3;return RotateZ90(i);}
template<> Vector3<int> &Vector3<int>::RotateX(const double r)
{const int i=((int)floor(r*2/PI+0.5))&3;return RotateX90(i);}
template<> Vector3<int> &Vector3<int>::RotateY(const double r)
{const int i=((int)floor(r*2/PI+0.5))&3;return RotateY90(i);}

//o
template<class T>
std::ostream &operator<<(std::ostream &_o,const Vector3<T> &_v)
{_o<<_v.x<<","<<_v.y<<","<<_v.z;return _o;}


/////////////////////////////
// NH[^jI

template<class T>
struct Quaternion
{
public:
	T x,y,z,w;
	typedef const T cT;
public:
	Quaternion(){}
	Quaternion(cT _x,cT _y,cT _z,cT _w):w(_w),x(_x),y(_y),z(_z){}
	Quaternion(const Vector3<T> &_v,cT _w):w(_w),x(_v.x),y(_v.y),z(_v.z){}
	Quaternion(const Quaternion &_q):w(_q.w),x(_q.x),y(_q.y),z(_q.z){}
	Quaternion(cT *_a):w(_a[3]),x(_a[0]),y(_a[1]),z(_a[2]){}
	//ϊ
	Quaternion(const Vector2<T> &_v);
	Quaternion(const Vector3<T> &_v);

	Quaternion &operator=(const Quaternion &_q){w=_q.w;x=_q.x;y=_q.y;z=_q.z;return *this;}

	void ToArray(cT *_a)const{_a[0]=x;_a[1]=y;_a[2]=z;_a[3]=w;}

	Quaternion &operator*=(const Quaternion &_q)
	{
		Vector3<double> v0(x,y,z),v1(_q.x,_q.y,_q.z),v;
		v=v0.ExProd(v1)+v1*w+v0*_q.w;x=v.x;y=v.y;z=v.z;
		w=w*_q.w-v0.DotProd(v1);
		return *this;
	}
	Quaternion operator*(const Quaternion &_q)const
	{
		Quaternion q;
		Vector3<double> v0(x,y,z),v1(_q.x,_q.y,_q.z),v;
		v=v0.ExProd(v1)+v1*w+v0*_q.w;q.x=v.x;q.y=v.y;q.z=v.z;
		q.w=w*_q.w-v0.DotProd(v1);
		return q;
	}

	Quaternion &operator+=(const Quaternion &_q){w+=_q.w;x+=_q.x;y+=_q.y;z+=_q.z;return *this;}
	Quaternion operator+(const Quaternion &_q)const{return Quaternion<T>(x+_q.x,y+_q.y,z+_q.z,w+_q.w);}
	Quaternion &operator-=(const Quaternion &_q){w-=_q.w;x-=_q.x;y-=_q.y;z-=_q.z;return *this;}
	Quaternion operator-(const Quaternion &_q)const{return Quaternion<T>(x-_q.x,y-_q.y,z-_q.z,w-_q.w);}
	Quaternion &operator*=(cT _m){w*=_m;x*=_m;y*=_m;z*=_m;return *this;}
	Quaternion operator*(cT _m)const{return Quaternion<T>(x*_m,y*_m,z*_m,w*_m);}
	Quaternion operator-()const{return Quaternion<T>(-x,-y,-z,-w);}

	Quaternion &Identity(){w=1;x=0;y=0;z=0;return *this;}

	//_v_thWA]NH[^jI
	//not for int
	Quaternion &RotationAxis(const Vector3<T> &_v,double _th)
	{
		Vector3<T> v=_v.Norm()*sin(_th/2);
		w=cos(_th/2);x=v.x;y=v.y;z=v.z;
		return *this;
	}

	Vector3<T> Rotate(const Vector3<T> &_v)const
	{
		Quaternion cj=Conj();
		Quaternion v(_v,0);
		Quaternion r=(*this)*v*cj;
		return Vector3<T>(r.x,r.y,r.z);
	}

	//
	Quaternion Conj()const{return Quaternion<T>(-x,-y,-z,w);}
	Quaternion &Conjugate(){x=-x;y=-y;z=-z;return *this;}

	//
	T Length2()const{return w*w+x*x+y*y+z*z;}
	double Length()const{return sqrt(w*w+x*x+y*y+z*z);}

	//t
	//not for int
	Quaternion operator~()const{return Inv();}
	Quaternion Inv()const{double l=Length2();return Quaternion<T>(-x/l,-y/l,-z/l,w/l);}
	Quaternion &Invert(){double l=Length2();w=w/l;x=-x/l;y=-y/l;z=-z/l;return *this;}

	//K
	//not for int
	Quaternion Norm()const{double l=Length();if(l==0){return Quaternion<T>(1,0,0,0);}return Quaternion<T>(x/l,y/l,z/l,w/l);}
	Quaternion &Normalize(){double l=Length();if(l==0){x=1;y=0;z=0;w=0;return *this;}w=w/l;x=x/l;y=y/l;z=z/l;return *this;}

	//
	T DotProd(const Quaternion &_q)const{return w*_q.w+x*_q.x+y*_q.y+z*_q.z;}

	//Ԋ֐
	//not for int
	
	//`
	static Quaternion Lerp(const Quaternion &_a,const Quaternion &_b,const double _t)
	{
		return _a+(_b-_a)*_t;
	}
	Quaternion Lerp(const Quaternion &_q,const double _t)const
	{
		return *this+(_q-*this)*_t;
	}
	//ʐ`
	static Quaternion Slerp(const Quaternion &_a,const Quaternion &_b,const double _t)
	{
		Quaternion tmpq;
		T dot=_a.DotProd(_b);
		if(dot<0){dot=-dot;tmpq=_b*-1;}
		else{tmpq=_b;}
		if(dot<0.95)
		{
		double th=acos(dot);
		return (  _a*sin(th*(1-_t)) + tmpq*sin(th*_t)  )*(1/sin(th));
		}
		else
		{
			return Lerp(_a,tmpq,_t);
		}
	}
	Quaternion Slerp(const Quaternion &_q,const double _t)const
	{
		Quaternion tmpq;
		T dot=DotProd(_q);
		if(dot<0){dot=-dot;tmpq=_q*-1;}
		else{tmpq=_q;}
		if(dot<0.95)
		{
			double th=acos(dot);
			return (  (*this)*sin(th*(1-_t)) + tmpq*sin(th*_t)  )*(1/sin(th));
		}
		else
		{
			return Lerp(tmpq,_t);
		}
	}
	//]Ȃʐ`
	static Quaternion SlerpNoInv(const Quaternion &_a,const Quaternion &_b,const double _t)
	{
		T dot=_a.DotProd(_b);
		if(dot>-0.95&&dot<0.95)
		{
		double th=acos(dot);
		return (  _a*sin(th*(1-_t)) + _b*sin(th*_t)  )*(1/sin(th));
		}
		else
		{
			return Lerp(_a,_b,_t);
		}
	}
	Quaternion SlerpNoInv(const Quaternion &_q,const double _t)const
	{
		T dot=DotProd(_q);
		if(dot>-0.95&&dot<0.95)
		{
			double th=acos(dot);
			return (  (*this)*sin(th*(1-_t)) + _q*sin(th*_t)  )*(1/sin(th));
		}
		else
		{
			return Lerp(_q,_t);
		}
	}
	//2
	static Quaternion Squad
	(const Quaternion &_a,const Quaternion &_b,const Quaternion &_c,const Quaternion &_d,const double _t)
	{
		return SlerpNoInv(SlerpNoInv(_a,_d,_t),SlerpNoInv(_b,_c,_t),2*_t*(1-_t));
	}
	Quaternion Squad
	(const Quaternion &_b,const Quaternion &_c,const Quaternion &_d,const double _t)const
	{
		return SlerpNoInv(*this,_d,_t).SlerpNoInv(SlerpNoInv(_b,_c,_t),2*_t*(1-_t));
	}

	//w(NH[^jI w=0 Ɍ)
	Quaternion Exp()const
	{
		const double th=sqrt(x*x+y*y+z*z),c=cos(th);
		if(th>0)
		{
			double sc=sin(th)/th;
			return Quaternion<T>(x*sc,y*sc,z*sc,c);
		}
		else
		{
			return Quaternion<T>(0,0,0,c);
		}
	}
	static Quaternion Exp(const Quaternion &_q)
	{
		const double th=sqrt(_q.x*_q.x+_q.y*_q.y+_q.z*_q.z),c=cos(th);
		if(th>0){const double sc=sin(th)/th;return Quaternion<T>(_q.x*sc,_q.y*sc,_q.z*sc,c);}
		else{return Quaternion<T>(0,0,0,c);}
	}
	//ΐ(PʃNH[^jI l=0 Ɍ)
	Quaternion Ln()const
	{
		const double th=acos(w);
		const double s=sin(th);
		if(s>0)
		{
			double sc=th/s;
			return Quaternion<T>(x*sc,y*sc,z*sc,0);
		}
		else
			return Quaternion<T>(0,0,0,0);
	}
	static Quaternion Ln(const Quaternion &_q)
	{
		const double th=acos(_q.w),s=sin(th);
		const double th2=atan2(sqrt(_q.x*_q.x+_q.y*_q.y+_q.z*_q.z),_q.w);
		const double l=_q.Length();
		if(s>0){double sc=th/s;return Quaternion<T>(_q.x*sc,_q.y*sc,_q.z*sc,0);}
		else{return Quaternion<T>(0,0,0,0);}
	}
	//XvCԗp
	// q0 q1 q2 q3āA
	// a=Spline(q0,q1,q2)
	// b=Spline(q1,q2,q3)
	// result=Squad(q0,a,b,q3,t)
	//ƂӂɎg
	static Quaternion Spline(const Quaternion &_m,const Quaternion &_0,const Quaternion &_p)
	{
		const Quaternion inv=_0.Conj();
		return _0*Exp(  (Ln(inv*_m)+Ln(inv*_p)) * -0.25  );
	}

	//]sɕϊ
	Matrix33<T> ToRotMatrix33()const
	{
		return Matrix33<T>(
			1-2*y*y-2*z*z,	2*x*y+2*w*z,	2*x*z-2*w*y,
			2*x*y-2*w*z,	1-2*x*x-2*z*z,	2*y*z+2*w*x,
			2*x*z+2*w*y,	2*y*z-2*w*x,	1-2*x*x-2*y*y
			);
	}
	
	//xWCgx]ɕ
	//q==joint*RotateAxis(Vector3(1,0,0),rot)
	//񂷉]͂Ȃ͂B
	void SplitJointRot(Quaternion<T> &joint,T &rot)const
	{
		Vector3<T> xe(1,0,0);
		Vector3<T> xdash=Rotate(xe);
		Vector3<T> n=xe.ExProd(xdash);
		n.Normalize();
		T cs=xe.DotProd(xdash);
		double ss=sqrt((1-cs)*0.5);
		double cc=sqrt((1+cs)*0.5);

		joint=Quaternion<T>(n.x*ss,n.y*ss,n.z*ss,cc);
		
		Quaternion<T> xrot=joint.Inv()*(*this);
		rot=atan2(xrot.x,xrot.w)*2;
	}
	//Saxisɂꍇ
	//q==joint*RotateAxis(axis,rot)
	void SplitJointRot(const Vector3<T> &axis,Quaternion<T> &joint,T &rot)const
	{
		Vector3<T> xe=axis;
		xe.Normalize();
		Vector3<T> xdash=Rotate(xe);
		Vector3<T> n=xe.ExProd(xdash);
		n.Normalize();
		T cs=xe.DotProd(xdash);
		double ss=sqrt((1-cs)*0.5);
		double cc=sqrt((1+cs)*0.5);

		joint=Quaternion<T>(n.x*ss,n.y*ss,n.z*ss,cc);
		
		Quaternion<T> xrot=joint.Inv()*(*this);
		rot=atan2(xrot.x,xrot.w)*2;
	}
};

//o
template<class T>
std::ostream &operator<<(std::ostream &_o,const Quaternion<T> &_q)
{_o<<_q.x<<","<<_q.y<<","<<_q.z<<","<<_q.w;return _o;}


//////////////////////////////
// ϊ
//

//xNg𖳗葼̃xNgɕϊB
template<class T> Vector2<T>::Vector2(const Vector3<T> &_v):x(_v.x),y(_v.y){}
template<class T> Vector2<T>::Vector2(const Quaternion<T> &_v):x(_v.x),y(_v.y){}
template<class T> Vector3<T>::Vector3(const Vector2<T> &_v):x(_v.x),y(_v.y),z(0){}
template<class T> Vector3<T>::Vector3(const Quaternion<T> &_v):x(_v.x),y(_v.y),z(_v.z){}
template<class T> Quaternion<T>::Quaternion(const Vector2<T> &_v):x(_v.x),y(_v.y),z(0),w(0){}
template<class T> Quaternion<T>::Quaternion(const Vector3<T> &_v):x(_v.x),y(_v.y),z(_v.z),w(0){}


/////////////////////////////
// 2x2s
//
template<class T>
struct Matrix22
{
public:
	T m[2][2];
	typedef const T cT;
public:
	Matrix22(){}
	Matrix22(cT _00,cT _01,cT _10,cT _11)
	{
		m[0][0]=_00;m[0][1]=_01;
		m[1][0]=_10;m[1][1]=_11;
	}
	Matrix22(const Vector2<T> &_v0,const Vector2<T> &_v1)
	{
		m[0][0]=_v0.x;m[0][1]=_v1.x;
		m[1][0]=_v0.y;m[1][1]=_v1.y;
	}
	Matrix22(const Matrix22 &_M)
	{
		*this=_M;
	}
	//Pʍs
	Matrix22 &Identity()
	{
		m[0][0]=1;m[0][1]=0;
		m[1][0]=0;m[1][1]=1;
		return *this;
	}
	//[s
	Matrix22 &Zero()
	{
		m[0][0]=0;m[0][1]=0;
		m[1][0]=0;m[1][1]=0;
		return *this;
	}
	
	Matrix22 &operator+=(const Matrix22 &_M)
	{
		for(int i=0;i<4;i++){int a=i/2,b=i%2;m[a][b]+=_M.m[a][b];}
		return *this;
	}
	Matrix22 &operator-=(const Matrix22 &_M)
	{
		for(int i=0;i<4;i++){int a=i/2,b=i%2;m[a][b]-=_M.m[a][b];}
		return *this;
	}
	Matrix22 &operator*=(const Matrix22 &_M)
	{
		Matrix22 t;
		for(int i=0;i<2;i++)
		{
			for(int j=0;j<2;j++)
			{
				t.m[i][j]=m[i][0]*_M.m[0][j]+m[i][1]*_M.m[1][j];
			}
		}
		*this=t;
		return *this;
	}

	Matrix22 operator+(const Matrix22 &_M)const
	{
		Matrix22 t;
		for(int i=0;i<4;i++){int a=i/2,b=i%2;t.m[a][b]=m[a][b]+_M.m[a][b];}
		return t;
	}
	Matrix22 operator-(const Matrix22 &_M)const
	{
		Matrix22 t;
		for(int i=0;i<4;i++){int a=i/2,b=i%2;t.m[a][b]=m[a][b]-_M.m[a][b];}
		return t;
	}
	Matrix22 operator-()const
	{
		Matrix22 t;
		for(int i=0;i<4;i++){int a=i/2,b=i%2;t.m[a][b]=-m[a][b];}
		return t;
	}
	Matrix22 operator*(const Matrix22 &_M)const
	{
		Matrix22 t;
		for(int i=0;i<2;i++)
		{
			for(int j=0;j<2;j++)
			{
				t.m[i][j]=m[i][0]*_M.m[0][j]+m[i][1]*_M.m[1][j];
			}
		}
		return t;
	}
	Vector2<T> operator*(const Vector2<T> &_v)const
	{
		return Vector2<T>(
			m[0][0]*_v.x+m[0][1]*_v.y,
			m[1][0]*_v.x+m[1][1]*_v.y
		);
	}
	Matrix22 operator*(cT _m)const
	{
		Matrix22 t;
		for(int i=0;i<4;i++){int a=i/2,b=i%2;t.m[a][b]=m[a][b]*_m;}
		return t;
	}
	
	void ToArray44(T *_a)const
	{
		int n=0;
		for(int i=0;i<2;i++)
		{
			for(int j=0;j<2;j++,n++)
			{
				_a[n]=m[i][j];
			}
			_a[n]=0;n++;
			_a[n]=0;n++;
		}	
		_a[n]=0;n++;
		_a[n]=0;n++;
		_a[n]=1;n++;
		_a[n]=0;n++;
		
		_a[n]=0;n++;
		_a[n]=0;n++;
		_a[n]=0;n++;
		_a[n]=1;n++;
	}
	
	//]u
	Matrix22 Transp()const
	{
		Matrix22 M(*this);
		std::swap(M.m[1][0],M.m[0][1]);
		return M;
	}
	Matrix22 &Transpose()
	{
		std::swap<T>(m[1][0],m[0][1]);
		return *this;
	}
	
	//g[X
	T Trace()const
	{
		return m[0][0]+m[1][1];
	}
	
	//s
	T Determinant()const
	{
		return
			+m[0][0]*m[1][1]
			-m[0][1]*m[1][0];
	}
	
	//]q
	T Cofactor(const int _i,const int _j)const
	{
		const int sig=((_i+_j)&1)?-1:1;
		return sig*m[1-_i][1-_j];
	}
	
	//ts
	//not for int
	Matrix22 operator~()const{reutrn Inv();}
	Matrix22 Inv()const
	{
		Matrix22 t;
		cT d=Determinant();
		for(int i=0;i<2;i++)
		{
			for(int j=0;j<2;j++)
			{
				t.m[i][j]=Cofactor(j,i)/d;
			}
		}
		return t;
	}
	Matrix22 &Invert()
	{
		Matrix22 t;
		cT d=Determinant();
		for(int i=0;i<2;i++)
		{
			for(int j=0;j<2;j++)
			{
				t.m[i][j]=Cofactor(j,i)/d;
			}
		}
		*this=t;
		return *this;
	}
	
	// Ax=b
	//not for int
	Vector2<T> Solve(const Vector2<T> &_b)const
	{
		cT d=Determinant();
		if(d==0)return Vector2<T>(0,0);
		Vector2<T> r;
		r.x=_b.x*m[1][1]-m[0][1]*_b.y;
		r.y=m[0][0]*_b.y-_b.x*m[1][0];
		return r*(1/d);
	}
	
	//fɕϊ(]sɌ)
	Vector2<T> ToComplex()const
	{
		Vector2<T> v(m[0][0],m[0][1]);
		return v;
	}
};

//o
template<class T>
std::ostream &operator<<(std::ostream &_o,const Matrix22<T> &_m)
{
	_o<<_m.m[0][0]<<","<<_m.m[0][1]<<std::endl
	  <<_m.m[1][0]<<","<<_m.m[1][1]<<std::endl;
	return _o;
}



/////////////////////////////
// 3x3s

template<class T>
struct Matrix33
{
public:
	T m[3][3];
	typedef const T cT;
public:
	Matrix33(){}
	Matrix33(cT _00,cT _01,cT _02,cT _10,cT _11,cT _12,cT _20,cT _21,cT _22)
	{
		m[0][0]=_00;m[0][1]=_01;m[0][2]=_02;
		m[1][0]=_10;m[1][1]=_11;m[1][2]=_12;
		m[2][0]=_20;m[2][1]=_21;m[2][2]=_22;
	}
	Matrix33(const Vector3<T> &_v0,const Vector3<T> &_v1,const Vector3<T> &_v2)
	{
		//m[0][0]=_v0.x;m[0][1]=_v0.y;m[0][2]=_v0.z;
		//m[1][0]=_v1.x;m[1][1]=_v1.y;m[1][2]=_v1.z;
		//m[2][0]=_v2.x;m[2][1]=_v2.y;m[2][2]=_v2.z;
		m[0][0]=_v0.x;m[0][1]=_v1.x;m[0][2]=_v2.x;
		m[1][0]=_v0.y;m[1][1]=_v1.y;m[1][2]=_v2.y;
		m[2][0]=_v0.z;m[2][1]=_v1.z;m[2][2]=_v2.z;
	}
	Matrix33(const Matrix33 &_M)
	{
		*this=_M;
	}
	//Pʍs
	Matrix33 &Identity()
	{
		m[0][0]=1;m[0][1]=0;m[0][2]=0;
		m[1][0]=0;m[1][1]=1;m[1][2]=0;
		m[2][0]=0;m[2][1]=0;m[2][2]=1;
		return *this;
	}
	//[s
	Matrix33 &Zero()
	{
		m[0][0]=0;m[0][1]=0;m[0][2]=0;
		m[1][0]=0;m[1][1]=0;m[1][2]=0;
		m[2][0]=0;m[2][1]=0;m[2][2]=0;
		return *this;
	}
	
	Matrix33 &operator+=(const Matrix33 &_M)
	{
		for(int i=0;i<9;i++){int a=i/3,b=i%3;m[a][b]+=_M.m[a][b];}
		return *this;
	}
	Matrix33 &operator-=(const Matrix33 &_M)
	{
		for(int i=0;i<9;i++){int a=i/3,b=i%3;m[a][b]-=_M.m[a][b];}
		return *this;
	}
	Matrix33 &operator*=(const Matrix33 &_M)
	{
		Matrix33 t;
		for(int i=0;i<3;i++)
		{
			for(int j=0;j<3;j++)
			{
				t.m[i][j]=m[i][0]*_M.m[0][j]+m[i][1]*_M.m[1][j]+m[i][2]*_M.m[2][j];
			}
		}
		*this=t;
		return *this;
	}

	Matrix33 operator+(const Matrix33 &_M)const
	{
		Matrix33 t;
		for(int i=0;i<9;i++){int a=i/3,b=i%3;t.m[a][b]=m[a][b]+_M.m[a][b];}
		return t;
	}
	Matrix33 operator-(const Matrix33 &_M)const
	{
		Matrix33 t;
		for(int i=0;i<9;i++){int a=i/3,b=i%3;t.m[a][b]=m[a][b]-_M.m[a][b];}
		return t;
	}
	Matrix33 operator-()const
	{
		Matrix33 t;
		for(int i=0;i<9;i++){int a=i/3,b=i%3;t.m[a][b]=-m[a][b];}
		return t;
	}
	Matrix33 operator*(const Matrix33 &_M)const
	{
		Matrix33 t;
		for(int i=0;i<3;i++)
		{
			for(int j=0;j<3;j++)
			{
				t.m[i][j]=m[i][0]*_M.m[0][j]+m[i][1]*_M.m[1][j]+m[i][2]*_M.m[2][j];
			}
		}
		return t;
	}
	Vector3<T> operator*(const Vector3<T> &_v)const
	{
		return Vector3<T>(
			m[0][0]*_v.x+m[0][1]*_v.y+m[0][2]*_v.z,
			m[1][0]*_v.x+m[1][1]*_v.y+m[1][2]*_v.z,
			m[2][0]*_v.x+m[2][1]*_v.y+m[2][2]*_v.z
		);
	}
	Matrix33 operator*(cT _m)const
	{
		Matrix33 t;
		for(int i=0;i<9;i++){int a=i/3,b=i%3;t.m[a][b]=m[a][b]*_m;}
		return t;
	}
	
	void ToArray44(T *_a)const
	{
		int n=0;
		for(int i=0;i<3;i++)
		{
			for(int j=0;j<3;j++,n++)
			{
				_a[n]=m[i][j];
			}
			_a[n]=0;n++;
		}
		_a[n]=0;n++;
		_a[n]=0;n++;
		_a[n]=0;n++;
		_a[n]=1;n++;
	}
	
	//]u
	Matrix33 Transp()const
	{
		Matrix33 M(*this);
		std::swap(M.m[1][0],M.m[0][1]);
		std::swap(M.m[1][2],M.m[2][1]);
		std::swap(M.m[2][0],M.m[0][2]);
		return M;
	}
	Matrix33 &Transpose()
	{
		std::swap<T>(m[1][0],m[0][1]);
		std::swap<T>(m[1][2],m[2][1]);
		std::swap<T>(m[2][0],m[0][2]);
		return *this;
	}
	
	//g[X
	T Trace()const
	{
		return m[0][0]+m[1][1]+m[2][2];
	}
	
	//s
	T Determinant()const
	{
		return
			+m[0][0]*m[1][1]*m[2][2]
			+m[0][1]*m[1][2]*m[2][0]
			+m[0][2]*m[1][0]*m[2][1]
			-m[0][2]*m[1][1]*m[2][0]
			-m[0][1]*m[1][0]*m[2][2]
			-m[0][0]*m[1][2]*m[2][1];
	}
	
	//]q
	T Cofactor(int _i,int _j)const
	{
		int a[2],b[2];
		for(int i=0,j=0;i<3;i++){if(_i==i)continue;a[j]=i;j++;}
		for(int i=0,j=0;i<3;i++){if(_j==i)continue;b[j]=i;j++;}
		const int sig=((_i+_j)&1)?-1:1;
		
		return (m[a[0]][b[0]]*m[a[1]][b[1]]-m[a[0]][b[1]]*m[a[1]][b[0]])*sig;
	}
	
	//ts
	//not for int
	Matrix33 operator~()const{reutrn Inv();}
	Matrix33 Inv()const
	{
		Matrix33 t;
		cT d=Determinant();
		for(int i=0;i<3;i++)
		{
			for(int j=0;j<3;j++)
			{
				t.m[i][j]=Cofactor(j,i)/d;
			}
		}
		return t;
	}
	Matrix33 &Invert()
	{
		Matrix33 t;
		cT d=Determinant();
		for(int i=0;i<3;i++)
		{
			for(int j=0;j<3;j++)
			{
				t.m[i][j]=Cofactor(j,i)/d;
			}
		}
		*this=t;
		return *this;
	}
	
	// Ax=b
	//not for int
	Vector3<T> Solve(const Vector3<T> &_b)const
	{
		cT d=Determinant();
		if(d==0)return Vector3<T>(0,0,0);
		Vector3<T> r;
		r.x=Matrix33<T>
		(   _b.x,m[0][1],m[0][2],
		    _b.y,m[1][1],m[1][2],
		    _b.z,m[2][1],m[2][2]).Determinant();
		r.y=Matrix33<T>
		(m[0][0],   _b.x,m[0][2],
		 m[1][0],   _b.y,m[1][2],
		 m[2][0],   _b.z,m[2][2]).Determinant();
		r.z=Matrix33<T>
		(m[0][0],m[0][1],   _b.x,
		 m[1][0],m[1][1],   _b.y,
		 m[2][0],m[2][1],   _b.z).Determinant();
		return r*(1/d);
	}
	
	//NH[^jIɕϊB(]sɌ)
	//not for int
	Quaternion<T> ToQuaternion()const
	{
		cT tr=Trace();
		if(tr>0)
		{
			Quaternion<T> q;
			q.w=sqrt(tr+1)/2;
			q.x=(m[1][2]-m[2][1])/(4*q.w);
			q.y=(m[2][0]-m[0][2])/(4*q.w);
			q.z=(m[0][1]-m[1][0])/(4*q.w);
			return q;
		}
		if(m[0][0]>=m[1][1]&&m[0][0]>=m[2][2])
		{
			Quaternion<T> q;
			q.x=sqrt(m[0][0]-m[1][1]-m[2][2]+1)/2;
			q.w=(m[1][2]-m[2][1])/(4*q.x);
			q.y=(m[0][1]+m[1][0])/(4*q.x);
			q.z=(m[0][2]+m[2][0])/(4*q.x);
			return q;
		}
		if(m[1][1]>=m[0][0]&&m[1][1]>=m[2][2])
		{
			Quaternion<T> q;
			q.y=sqrt(m[1][1]-m[0][0]-m[2][2]+1)/2;
			q.w=(m[2][0]-m[0][2])/(4*q.y);
			q.x=(m[0][1]+m[1][0])/(4*q.y);
			q.z=(m[1][2]+m[2][1])/(4*q.y);
			return q;
		}
		{
			Quaternion<T> q;
			q.z=sqrt(m[2][2]-m[0][0]-m[1][1]+1)/2;
			q.w=(m[0][1]-m[1][0])/(4*q.z);
			q.x=(m[2][0]+m[0][2])/(4*q.z);
			q.y=(m[2][1]+m[1][2])/(4*q.z);
			return q;
		}
	}
};

//o
template<class T>
std::ostream &operator<<(std::ostream &_o,const Matrix33<T> &_m)
{
	_o<<_m.m[0][0]<<","<<_m.m[0][1]<<","<<_m.m[0][2]<<std::endl
	  <<_m.m[1][0]<<","<<_m.m[1][1]<<","<<_m.m[1][2]<<std::endl
	  <<_m.m[2][0]<<","<<_m.m[2][1]<<","<<_m.m[2][2];
	return _o;
}



/////////////////////////////
// 4x4s

template<class T>
struct Matrix44
{
public:
	T m[4][4];
	typedef const T cT;
public:
	Matrix44(){}
	Matrix44(
		cT _00,cT _01,cT _02,cT _03,
		cT _10,cT _11,cT _12,cT _13,
		cT _20,cT _21,cT _22,cT _23,
		cT _30,cT _31,cT _32,cT _33
		)
	{
		m[0][0]=_00;m[0][1]=_01;m[0][2]=_02;m[0][3]=_03;
		m[1][0]=_10;m[1][1]=_11;m[1][2]=_12;m[1][3]=_13;
		m[2][0]=_20;m[2][1]=_21;m[2][2]=_22;m[2][3]=_23;
		m[3][0]=_30;m[3][1]=_31;m[3][2]=_32;m[3][3]=_33;
	}
	Matrix44(const Vector3<T> &_v0,const Vector3<T> &_v1,const Vector3<T> &_v2,const Vector3<T> &_t)
	{
/*		m[0][0]=_v0.x;m[0][1]=_v1.x;m[0][2]=_v2.x;m[0][3]=0;
		m[1][0]=_v0.y;m[1][1]=_v1.y;m[1][2]=_v2.y;m[0][3]=0;
		m[2][0]=_v0.z;m[2][1]=_v1.z;m[2][2]=_v2.z;m[0][3]=0;
		m[3][0]= _t.z;m[3][1]= _t.z;m[3][2]= _t.z;m[3][3]=1;*/
		m[0][0]=_v0.x;m[0][1]=_v1.x;m[0][2]=_v2.x;m[0][3]=_t.x;
		m[1][0]=_v0.y;m[1][1]=_v1.y;m[1][2]=_v2.y;m[1][3]=_t.y;
		m[2][0]=_v0.z;m[2][1]=_v1.z;m[2][2]=_v2.z;m[2][3]=_t.z;
		m[3][0]=0;    m[3][1]=0;    m[3][2]=0;    m[3][3]=1;
	}
	Matrix44(const Matrix44 &_M)
	{
		*this=_M;
	}
	//Pʍs
	Matrix44 &Identity()
	{
		m[0][0]=1;m[0][1]=0;m[0][2]=0;m[0][3]=0;
		m[1][0]=0;m[1][1]=1;m[1][2]=0;m[1][3]=0;
		m[2][0]=0;m[2][1]=0;m[2][2]=1;m[2][3]=0;
		m[3][0]=0;m[3][1]=0;m[3][2]=0;m[3][3]=1;
		return *this;
	}
	//[s
	Matrix44 &Zero()
	{
		m[0][0]=0;m[0][1]=0;m[0][2]=0;m[0][3]=0;
		m[1][0]=0;m[1][1]=0;m[1][2]=0;m[1][3]=0;
		m[2][0]=0;m[2][1]=0;m[2][2]=0;m[2][3]=0;
		m[3][0]=0;m[3][1]=0;m[3][2]=0;m[3][3]=0;
		return *this;
	}
	Matrix44 &Set(cT _m[16])
	{
		for(int i=0;i<16;i++){int a=i/4,b=i%4;m[a][b]=_m[i];}
		return *this;
	}
	
	Matrix44 &operator+=(const Matrix44 &_M)
	{
		for(int i=0;i<16;i++){int a=i/4,b=i%4;m[a][b]+=_M.m[a][b];}
		return *this;
	}
	Matrix44 &operator-=(const Matrix44 &_M)
	{
		for(int i=0;i<16;i++){int a=i/4,b=i%4;m[a][b]-=_M.m[a][b];}
		return *this;
	}
	Matrix44 &operator*=(const Matrix44 &_M)
	{
		Matrix44 t;
		for(int i=0;i<4;i++)
		{
			for(int j=0;j<4;j++)
			{
				t.m[i][j]=
					m[i][0]*_M.m[0][j]+
					m[i][1]*_M.m[1][j]+
					m[i][2]*_M.m[2][j]+
					m[i][3]*_M.m[3][j];
			}
		}
		*this=t;
		return *this;
	}
	Matrix44 &operator*=(cT _m)
	{
		for(int i=0;i<16;i++){int a=i/4,b=i%4;m[a][b]*=m;}
		return *this;
	}

	Matrix44 operator+(const Matrix44 &_M)const
	{
		Matrix44 t;
		for(int i=0;i<16;i++){int a=i/4,b=i%4;t.m[a][b]=m[a][b]+_M.m[a][b];}
		return t;
	}
	Matrix44 operator-(const Matrix44 &_M)const
	{
		Matrix44 t;
		for(int i=0;i<16;i++){int a=i/4,b=i%4;t.m[a][b]=m[a][b]-_M.m[a][b];}
		return t;
	}
	Matrix44 operator-()const
	{
		Matrix44 t;
		for(int i=0;i<16;i++){int a=i/4,b=i%4;t.m[a][b]=-m[a][b];}
		return t;
	}
	Matrix44 operator*(const Matrix44 &_M)const
	{
		Matrix44 t;
		for(int i=0;i<4;i++)
		{
			for(int j=0;j<4;j++)
			{
				t.m[i][j]=
					m[i][0]*_M.m[0][j]+
					m[i][1]*_M.m[1][j]+
					m[i][2]*_M.m[2][j]+
					m[i][3]*_M.m[3][j];
			}
		}
		return t;
	}
	Matrix44 operator*(cT _m)const
	{
		Matrix44 t;
		for(int i=0;i<16;i++){int a=i/4,b=i%4;t.m[a][b]=m[a][b]*_m;}
		return t;
	}
	
	Vector3<T> operator*(const Vector3<T> _v)const
	{
/*		Vector3<T> r(
			m[0][0]*_v.x+m[0][1]*_v.y+m[0][2]*_v.z,
			m[1][0]*_v.x+m[1][1]*_v.y+m[1][2]*_v.z,
			m[2][0]*_v.x+m[2][1]*_v.y+m[2][2]*_v.z
		);
		r.x+=m[3][0];
		r.y+=m[3][1];
		r.z+=m[3][2];
		return r;*/
		return Vector3<T>(
			m[0][0]*_v.x+m[0][1]*_v.y+m[0][2]*_v.z+m[0][3],
			m[1][0]*_v.x+m[1][1]*_v.y+m[1][2]*_v.z+m[1][3],
			m[2][0]*_v.x+m[2][1]*_v.y+m[2][2]*_v.z+m[2][3]
		);
	}
	
	void ToArray44(T *_a)const
	{
		int n=0;
		for(int i=0;i<4;i++)
		{
			for(int j=0;j<4;j++,n++)
			{
				_a[n]=m[i][j];
			}
		}
	}
	
	//]u
	Matrix44 Transp()const
	{
		Matrix44 M(*this);
		std::swap(M.m[1][0],M.m[0][1]);
		std::swap(M.m[2][0],M.m[0][2]);
		std::swap(M.m[3][0],M.m[0][3]);
		std::swap(M.m[2][1],M.m[1][2]);
		std::swap(M.m[3][1],M.m[1][3]);
		std::swap(M.m[3][2],M.m[2][3]);

		return M;
	}
	Matrix44 &Transpose()
	{
		std::swap(m[1][0],m[0][1]);
		std::swap(m[2][0],m[0][2]);
		std::swap(m[3][0],m[0][3]);
		std::swap(m[2][1],m[1][2]);
		std::swap(m[3][1],m[1][3]);
		std::swap(m[3][2],m[2][3]);
		return *this;
	}
	
	//g[X
	T Trace()const
	{
		return m[0][0]+m[1][1]+m[2][2]+m[3][3];
	}

	Matrix44 &Scale(const Vector3<T> &s)
	{
		m[0][0]=s.x;m[0][1]=0;m[0][2]=0;m[0][3]=0;
		m[1][0]=0;m[1][1]=s.y;m[1][2]=0;m[1][3]=0;
		m[2][0]=0;m[2][1]=0;m[2][2]=s.z;m[2][3]=0;
		m[3][0]=0;m[3][1]=0;m[3][2]=0;m[3][3]=1;
		return *this;
	}
	Matrix44 &Translate(const Vector3<T> &t)
	{
/*		m[0][0]=1;m[0][1]=0;m[0][2]=0;m[0][3]=0;
		m[1][0]=0;m[1][1]=1;m[1][2]=0;m[1][3]=0;
		m[2][0]=0;m[2][1]=0;m[2][2]=1;m[2][3]=0;
		m[3][0]=t.x;m[3][1]=t.y;m[3][2]=t.z;m[3][3]=1;*/
		m[0][0]=1;m[0][1]=0;m[0][2]=0;m[0][3]=t.x;
		m[1][0]=0;m[1][1]=1;m[1][2]=0;m[1][3]=t.y;
		m[2][0]=0;m[2][1]=0;m[2][2]=1;m[2][3]=t.z;
		m[3][0]=0;m[3][1]=0;m[3][2]=0;m[3][3]=1;
		return *this;
	}

	//not for int
	Matrix44 &LookAt(const Vector3<T> &eye,const Vector3<T> &look,const Vector3<T> &sky)
	{
		Vector3<T> za=(eye-look).Norm();
		Vector3<T> xa=(sky.ExProd(za)).Norm();
		Vector3<T> ya=za.ExProd(xa);

/*		T mat[16]={
			xa.x,ya.x,za.x,0,
			xa.y,ya.y,za.y,0,
			xa.z,ya.z,za.z,0,
			-xa.DotProd(eye),-ya.DotProd(eye),-za.DotProd(eye),1
		};*/
		T mat[16]={
			xa.x,ya.x,za.x,-xa.DotProd(eye),
			xa.y,ya.y,za.y,-ya.DotProd(eye),
			xa.z,ya.z,za.z,-za.DotProd(eye),
			0,0,0,1
		};
		Set(mat);
		return *this;
	}
	Matrix44 &Perspective(const T angle,const T aspect,const T zn,const T zf)
	{
		cT H=2*tan(angle*3.141592/180/2);
		cT W=H*aspect;

		cT mat[16]={
			2/W,0,0,0,
			0,2/H,0,0,
			0,0,-2*zf/(zf-zn)+1,-1,
			0,0,-2*zf*zn/(zf-zn),0
		};
		Set(mat);
		return *this;
	}

};

//o
template<class T>
std::ostream &operator<<(std::ostream &_o,const Matrix44<T> &_m)
{
	_o<<_m.m[0][0]<<","<<_m.m[0][1]<<","<<_m.m[0][2]<<","<<_m.m[0][3]<<std::endl
	  <<_m.m[1][0]<<","<<_m.m[1][1]<<","<<_m.m[1][2]<<","<<_m.m[1][3]<<std::endl
	  <<_m.m[2][0]<<","<<_m.m[2][1]<<","<<_m.m[2][2]<<","<<_m.m[2][3]<<std::endl
	  <<_m.m[3][0]<<","<<_m.m[3][1]<<","<<_m.m[3][2]<<","<<_m.m[3][3]<<std::endl;
	return _o;
}

/////////////////////////////
// 2`̈

template<class T> struct Rect2
{
public:
	Vector2<T> p1,p2;
	typedef const T cT;
public:
	Rect2<T>(const Vector2<T> &_1,const Vector2<T> &_2):p1(_1),p2(_2){}
	Rect2<T>(const Rect2<T> &_r):p1(_r.p1),p2(_r.p2){}
	Rect2<T>(const T x1,const T y1,const T x2,const T y2):p1(x1,y1),p2(x2,y2){}
	Rect2<T>(){}
	//_v̈ɓĂtrueAĂȂfalse
	bool In(const Vector2<T> &_v)const
	{
		if(p1.x<=_v.x&&_v.x<=p2.x&&p1.y<=_v.y&&_v.y<=p2.y)
			return true;
		return false;
	}
	//_rSɗ̈ɓĂtrueAłȂfalse
	bool In(const Rect2<T> &_r)const
	{
		if(In(_r.p1)&&In(_r.p2))return true;
		return false;
	}
	//_rƏdȂ镔߂B
	//dȂĂresult.IsRectƂtrueɂȂB
	Rect2<T> Intersect(const Rect2<T> &_r)const
	{
		return Rect2<T>(
			std::max(p1.x,_r.p1.x),std::max(p1.y,_r.p1.y),
			std::min(p2.x,_r.p2.x),std::min(p2.y,_r.p2.y)
		);
	}
	//this_r\[gĂIntersectƂ
	Rect2<T> IntersectSort(const Rect2<T> &_r)const
	{
		Rect2<T> a=*this;b=_r;
		a.AscendingOrder();b.AscendingOrder();
		return a.Intersect(b);
	}
	bool IsIntersect(const Rect2<T> &_r)const
	{
		Rect2<T> a=*this,b=_r;
		a.AscendingOrder();b.AscendingOrder();
		if( (a.p2.x<b.p1.x&&a.p1.x<b.p2.x)||
			(a.p2.x>b.p1.x&&a.p1.x>b.p2.x) )
		{
			return false;
		}
		if( (a.p2.y<b.p1.y&&a.p1.y<b.p2.y)||
			(a.p2.y>b.p1.y&&a.p1.y>b.p2.y) )
		{
			return false;
		}
		return true;
	}
	//_v̈ɓ悤ɍLB
	void Insert(const Vector2<T> &_v)
	{
		if(p1.x>_v.x){p1.x=_v.x;}
		if(p2.x<_v.x){p2.x=_v.x;}
		if(p1.y>_v.y){p1.y=_v.y;}
		if(p2.y<_v.y){p2.y=_v.y;}
	}
	void Insert(const Rect2<T> &_r)
	{
		Insert(_r.p1);
		Insert(_r.p2);
	}
	Rect2<T> &operator+=(const Vector2<T> &_v)
	{
		Insert(_v);
		return *this;
	}
	Rect2<T> &operator+=(const Rect2<T> &_r)
	{
		Insert(_r);
		return *this;
	}
	Rect2<T> &Offset(const Vector2<T> &_v)
	{
		p1+=_v;p2+=_v;return *this;
	}
	Rect2<T> Ofs(const Vector2<T> &_v)const
	{
		return Rect2<T> (p1+_v,p2+_v);
	}
	//Ίp̃xNg
	Vector2<T> Diagonal()const{return p2-p1;}
	//̗̈0-1Ɖ肵ƂfŁÄ̗𕪂
	Rect2<T> Subdiv(const Rect2<T> &_r)const
	{
		Vector2f dthis=Diagonal();
		//Vector2f dsub=_r.Diagonal();

		return Rect2<T>(
			Vector2<T>(p1.x+dthis.x*_r.p1.x,p1.y+dthis.y*_r.p1.y),
			Vector2<T>(p1.x+dthis.x*_r.p2.x,p1.y+dthis.y*_r.p2.y));
	}
	//_v:0-1̗̈ɕϊ
	Vector2<T> TransformTo(const Vector2<T> &_v)const
	{
		Vector2<T> dv=_v-p1,dr=Diagonal();
		return Vector2<T>(dv.x/dr.x,dv.y/dr.y);
	}
	//Wlɕׂ
	void AscendingOrder()
	{
		if(p1.x>p2.x)
			std::swap(p1.x,p2.x);
		if(p1.y>p2.y)
			std::swap(p1.y,p2.y);
	}
	//WlȂtrueAłȂȂfalse
	bool IsRect()const
	{
		if(p1.x>p2.x)
			return false;
		if(p1.y>p2.y)
			return false;
		return true;
	}
};

//o
template<class T>
std::ostream &operator<<(std::ostream &_o,const Rect2<T> &_r)
{_o<<"("<<_r.p1<<"),("<<_r.p2<<")";return _o;}


/////////////////////////////
// 3`̈

template<class T> struct Rect3
{
public:
	Vector3<T> p1,p2;
public:
	Rect3<T>(const Vector3<T> &_1,const Vector3<T> &_2):p1(_1),p2(_2){}
	Rect3<T>(const Rect3<T> &_r):p1(_r.p1),p2(_r.p2){}
	Rect3<T>(const T x1,const T y1,const T z1,const T x2,const T y2,const T z2):p1(x1,y1,z1),p2(x2,y2,z2){}
	Rect3<T>(){}
	//_v̈ɓĂtrueAĂȂfalse
	bool In(const Vector3<T> &_v)const
	{
		if(p1.x<=_v.x&&_v.x<=p2.x&&
			p1.y<=_v.y&&_v.y<=p2.y&&
			p1.z<=_v.z&&_v.z<=p2.z)
			return true;
		return false;
	}
	//_rSɗ̈ɓĂtrueAłȂfalse
	bool In(const Rect3<T> &_r)const
	{
		if(In(_r.p1)&&In(_r.p2))return true;
		return false;
	}
	//_rƏdȂ镔߂B
	//Athis_rIsRectƂtrueɂȂȂƂȂB
	//dȂĂresult.IsRectƂtrueɂȂB
	Rect3<T> Intersect(const Rect2<T> &_r)const
	{
		return Rect3<T>(
			std::max(p1.x,_r.p1.x),std::max(p1.y,_r.p1.y),std::max(p1.z,_r.p1.z),
			std::min(p2.x,_r.p2.x),std::min(p2.y,_r.p2.y),std::min(p2.z,_r.p2.z)
		);
	}
	//this_r\[gĂIntersectƂ
	Rect3<T> IntersectSort(const Rect2<T> &_r)const
	{
		Rect3<T> a=*this;b=_r;
		a.AscendingOrder();b.AscendingOrder();
		return a.Intersect(b);
	}
	//_v̈ɓ悤ɍLB
	void Insert(const Vector3<T> &_v)
	{
		if(p1.x>_v.x){p1.x=_v.x;}
		if(p2.x<_v.x){p2.x=_v.x;}
		if(p1.y>_v.y){p1.y=_v.y;}
		if(p2.y<_v.y){p2.y=_v.y;}
		if(p1.z>_v.z){p1.z=_v.z;}
		if(p2.z<_v.z){p2.z=_v.z;}
	}
	//Ίp̃xNg
	Vector3<T> Diagonal()const{return p2-p1;}
	//̗̈0-1Ɖ肵ƂfŁÄ̗𕪂
	Rect3<T> Subdiv(const Rect3<T> &_r)const
	{
		Vector3f dthis=Diagonal();

		return Rect3<T>(
			Vector3<T>(p1.x+dthis.x*_r.p1.x,p1.y+dthis.y*_r.p1.y,p1.z+dthis.z*_r.p1.z),
			Vector3<T>(p1.x+dthis.x*_r.p2.x,p1.y+dthis.y*_r.p2.y,p1.z+dthis.z*_r.p2.z));
	}
	//_v:0-1̗̈ɕϊ
	Vector3<T> TransformTo(const Vector3<T> &_v)const
	{
		Vector3<T> dv=_v-p1,dr=Diagonal();
		return Vector3<T>(dv.x/dr.x,dv.y/dr.y,dv.z/dr.z);
	}
	//Wlɕׂ
	void AscendingOrder()
	{
		if(p1.x>p2.x)
			std::swap<T>(p1.x,p2.x);
		if(p1.y>p2.y)
			std::swap<T>(p1.y,p2.y);
		if(p1.z>p2.z)
			std::swap<T>(p1.z,p2.z);
	}
	//WlȂtrueAłȂȂfalse
	bool IsRect()const
	{
		if(p1.x>p2.x)
			return false;
		if(p1.y>p2.y)
			return false;
		if(p1.z>p2.z)
			return false;
		return true;
	}
};

//o
template<class T>
std::ostream &operator<<(std::ostream &_o,const Rect3<T> &_r)
{_o<<"("<<_r.p1<<"),("<<_r.p2<<")";return _o;}


}//end of namespace Rydot

#endif

