
#ifndef __MOTION_H__
#define __MOTION_H__

#include <matrix.h>
#include <matrixutil.h>
#include <vector>
#include <string>

namespace Rydot
{

typedef std::vector<int> intv;
typedef std::vector<Vector3f> Vector3fv;
typedef std::vector<Quaternionf> Quaternionfv; 

class IModelLoader;

class Model;

//////////////////////////////////////////////////
// Joint
// [V̕sړϊS
//
class Joint
{
public:
	std::string name;
	int link;
	int nscene;
	intv frame;
	Vector3fv position;
	int usespline;
public:
	Joint():usespline(1){}

	//p̈ʒuŕ⊮
	Vector3f Interpolate(float p,int loop,int end)
	{
		//p%=end;
		p=fmod(p,end);
		int a,b,i;
		for(i=0;i<nscene;i++)
		{
			if(frame[i]>p)
				break;
		}
		a=i-1;
		b=i;
		if(i==nscene)
		{
			if(loop)
				b=0;
			else
			{
				return position[i-1];
			}
		}
		int fa=frame[a],fb=frame[b];
		float ratio=(p-fa)/ ((end+fb-fa)%end);
		if(usespline)
			return CatmullRom
			(position[(a+nscene-1)%nscene],position[a],position[b],position[(b+1)%nscene],ratio);
		else
			//linear interpolate
			return position[a]*(1-ratio)+position[b]*(ratio);
	}
};


//////////////////////////////////////////////////
// Attitude
// [V̉]ϊS
//
class Attitude
{
public:
	std::string name;
	int link;
	int nscene;
	intv frame;
	Quaternionfv rotation;
	Quaternionfv splinecoef;
	int usespline;
public:
	Attitude():usespline(1){}
	void MakeSplineCoef()
	{
		if(nscene<=1)return;
		splinecoef.resize(nscene);
		for(int i=0;i<nscene;i++)
		{
			Quaternionf Q0=rotation[(i+nscene-1)%nscene],Q1=rotation[i],
			Q2=rotation[(i+1)%nscene];
//			Quaternionf q0=(Q0+Q1).Length2()<(Q0-Q1).Length2()?-Q0:Q0;
//			Quaternionf q2=(Q1+Q2).Length2()<(Q1-Q2).Length2()?-Q2:Q2;
//			Quaternionf q3=(Q2+Q3).Length2()<(Q2-Q3).Length2()?-Q3:Q3;
			splinecoef[i]=Quaternionf::Spline(Q0,Q1,Q2);
		}
	}
	Quaternionf Interpolate(float p,int loop,int end)
	{
		//p%=end;
		p=fmod(p,end);
		int a,b,i;
		for(i=0;i<nscene;i++)
		{
			if(frame[i]>p)
				break;
		}
		a=i-1;
		b=i;
		if(i==nscene)
		{
			if(loop)
				b=0;
			else
			{
				return rotation[i-1];
			}
		}
		int fa=frame[a],fb=frame[b];
		
		int w=(end+fb-fa)%end;
		float ratio=(p-fa)/ w;
		if(usespline)
		{
//			Quaternionf Q0=rotation[(a+nscene-1)%nscene],Q1=rotation[a],
//			Q2=rotation[b],Q3=rotation[(b+1)%nscene];
			Quaternionf Q1=rotation[a],Q2=rotation[b];
			Quaternionf A=splinecoef[a],B=splinecoef[b];
//			Quaternionf q0=(Q0+Q1).Length2()<(Q0-Q1).Length2()?-Q0:Q0;
//			Quaternionf q2=(Q1+Q2).Length2()<(Q1-Q2).Length2()?-Q2:Q2;
//			Quaternionf q3=(Q2+Q3).Length2()<(Q2-Q3).Length2()?-Q3:Q3;
//			Quaternionf A=Quaternionf::Spline(Q0,Q1,Q2);
//			Quaternionf B=Quaternionf::Spline(Q1,Q2,Q3);
			Quaternionf r=Quaternionf::Squad(Q1,A,B,Q2,ratio);
		//	r.Normalize();
			return r;
		}
		else
			//linear interpolate
			return Quaternionf::Slerp(rotation[a],rotation[b],ratio);
	}
};
typedef std::vector<Joint> Jointv;
typedef std::vector<Attitude> Attitudev;


//////////////////////////////////////////////////
// [V
// L[t[pă[VԂs
//
class Motion
{
public:
	std::string name;
	Jointv joints;
	Attitudev attitudes;
	int endframe;
	int loop;
public:
	
	void Append(Attitude &a){attitudes.push_back(a);}
	void Append(Joint &j){joints.push_back(j);}
};
typedef std::vector<Motion> Motionv;


//////////////////////////////////////////////////
// {[
//
class Bone
{
public:
	std::string name;
	int parent;// e{[ID
	intv child;// q{[ID
	intv linked_meshes;// ڊ֘AtĂ郁bVID
	int root,top,norm;
	Vector3f originalcoord;// Ƃ̍W
	Vector3f offset;// e{[̃ItZbg
	Quaternionf originalattitude;// Ƃ̎p
	Quaternionf rotation;// ]
public:
	Bone():parent(-1){}
};
typedef std::vector<Bone> Bonev;


//////////////////////////////////////////////////
// |[Y
//
class Pose
{
public:
	Bonev bones;//{[
	Motionv motions;//[V
	intv roots;//{[̃[g

//	static IModelLoader *loader;
public:
	Pose(){}

/*	bool Load(std::string filename)
	{
		return loader->Load(filename,*this);
	}*/
	
	void Append(Bone &b){bones.push_back(b);}
	void Append(Motion &m){motions.push_back(m);}
	void Append(int &r){roots.push_back(r);}

	//[V̖O烂[VIDT
	//Ȃ-1ԂB
	int GetMotionID(const std::string &name)const
	{
		int msz=motions.size();
		for(int i=0;i<msz;i++)
		{
			if(name==motions[i].name)
				return i;
		}
		return -1;
	}

	//{[̖O{[IDT
	//Ȃ-1ԂB
	int GetBoneID(std::string &name)
	{
		int bsz=bones.size();
		for(int i=0;i<bsz;i++)
		{
			if(name==bones[i].name)
				return i;
		}
		return -1;
	}

	//|[YƂ点B
	//(w肵[V̎w肵t[{[̎pɓKpB)
	//nmotion:[Vԍ
	//count:̃[V̈ʒuBmotions[nmotion].end𒴂0ɖ߂B
	void SetPose(int nmotion,float count)
	{
		Motion &m=motions[nmotion];
		int asz=m.attitudes.size();
		int rsz=roots.size();
		int loop=m.loop;
		int end=m.endframe;
		for(int i=0;i<asz;i++)
		{
			Attitude &a=m.attitudes[i];
			bones[a.link].rotation=a.Interpolate(count,loop,end);
		}
		for(int i=0;i<rsz;i++)
		{
			bones[roots[i]].offset=m.joints[0].Interpolate(count,loop,end);
		}
	}

	//݂̃|[YɁA䗦ratioŕʂ̃|[Y|킹
	//݂̃{[̎pɁAw肵[V̎w肵t[̎p킹B
	//DȂ|[Y悤B
	void InterpolatePose(int nmotion,float count,float ratio)
	{
		Motion &m=motions[nmotion];
		int asz=m.attitudes.size();
		int rsz=roots.size();
		int loop=m.loop;
		int end=m.endframe;
		for(int i=0;i<asz;i++)
		{
			Attitude &a=m.attitudes[i];
			bones[a.link].rotation=Quaternionf::Slerp(
				bones[a.link].rotation,
				a.Interpolate(count,loop,end),ratio);
		}
		for(int i=0;i<rsz;i++)
		{
			bones[roots[i]].offset=
				bones[roots[i]].offset*(1-ratio)+
				m.joints[0].Interpolate(count,loop,end)*(ratio);
		}
	}

	//݂bone[i]̎pvZ
	int CalcPresentAttitude(int i,Vector3f &coord,Quaternionf &atti)const
	{
		const Bone &b=bones[i];
		if(b.parent==-1)
		{
			coord=b.offset;
			atti=b.rotation;
			return 0;
		}
		CalcPresentAttitude(b.parent,coord,atti);
		coord+=atti.Rotate(b.offset);
		atti*=b.rotation;
		return 0;
	}

	//{[̍WAe{[̑΍Wɕϊ
	void MakeSubCoords(int n)
	{
		Bone &b=bones[n];
		int nc=b.child.size();
		for(int i=0;i<nc;i++)
		{
			int j=b.child[i];
			Bone &c=bones[j];
			if(c.norm==-1)continue;
			MakeSubCoords(j);
			c.offset-=b.offset;
			Quaternionf q=b.rotation.Inv();
			Vector3f v=q.Rotate(c.offset);
			c.offset=v;
			c.rotation=b.rotation.Inv()*c.rotation;
		}
	}

	void DbgBoneTree()
	{
		FILE *fp=fopen("dbgbonetree.txt","wt");

		for(int i=0;i<bones.size();i++)
		{
			if(bones[i].parent!=-1)
			{
			fprintf(fp,"%s(%d):[parent:%s(%d)]\n",
				bones[i].name.c_str(),i,
				bones[bones[i].parent].name.c_str(),bones[i].parent
				);
			}else
			{
				fprintf(fp,"%s(%d):\n",bones[i].name.c_str(),i);
			}
			for(int j=0;j<bones[i].child.size();j++)
			{
				fprintf(fp," %s(%d)\n",bones[bones[i].child[j]].name.c_str(),bones[i].child[j]);
			}
		}

		fprintf(fp,"\ntree\n");

		for(int i=0;i<roots.size();i++)
		{
			fprintf(fp,"root\n");
			DbgBone(fp,roots[i]);
		}

		fclose(fp);
	}

	void DbgBone(FILE *fp,int i)
	{
		fprintf(fp,"%s\n",bones[i].name.c_str());

		for(int j=0;j<bones[i].child.size();j++)
		{
			DbgBone(fp,bones[i].child[j]);
		}
	}
};

}//end of namespace Rydot

#endif

