
// Delauney Trangulation

#ifndef __DELAUNEY_H__
#define __DELAUNEY_H__

#include <vector>
#include <matrix.h>

namespace Rydot
{

class TriangleIndex
{
public:
	int p1,p2,p3;
	TriangleIndex(int a,int b,int c):p1(a),p2(b),p3(c){}
	TriangleIndex(){}
};

class EdgeIndex
{
public:
	int p1,p2;
	EdgeIndex(int a,int b):p1(a),p2(b){}
	EdgeIndex(){}
};

typedef std::vector< EdgeIndex > EIv;
typedef std::vector< TriangleIndex > TIv;

template <class T=double>
class DbgTris
{
public:
	typedef std::pair<Vector2<T>,int> PP;
	typedef std::vector< PP > PPv;
	TIv triangles;
	PPv points;
	EIv connections;
	DbgTris();
	DbgTris(const TIv &t,const PPv &p,const EIv &c):triangles(t),points(p),connections(c){}
};

template <class T=double>
class DelauneyTriangulator
{
	typedef std::pair<Vector2<T>,int> PP;
	typedef std::vector< PP > PPv;
	typedef std::vector< Vector2<T> > Pv;
	PPv points;
	EIv connections;
	TIv &triangles;
	//typedef std::vector< std::pair<TIv,PPv> > TIvPPvv;
	typedef std::vector<DbgTris<T> > DbgTrisv;
	DbgTrisv *dbg;
public:
	DelauneyTriangulator(const Pv &_points,
		const EIv &_connections,TIv &_triangles,DbgTrisv *_dbg=NULL)
	:connections(_connections),triangles(_triangles),dbg(_dbg)
	{
		if(_points.empty())return;

		points.resize(_points.size());
		for(int i=0;i<_points.size();++i)
		{
			points[i].first=_points[i];
			points[i].second=i;
		}

		int nv=points.size();
		//qsort((void*)&(points.front()),nv,sizeof(PP),Compare);

		//make inverse map
		std::vector<int> invmap(points.size());
		for(int i=0;i<points.size();++i)
		{
			invmap[points[i].second]=i;
		}

		for(int i=0;i<connections.size();++i)
		{
			connections[i].p1=invmap[connections[i].p1];
			connections[i].p2=invmap[connections[i].p2];
		}

		//_̃oEh`߂
		//Find the maximum and minimum vertex bounds.
		//This is to allow calculation of the bounding triangle
		Rect2<T> minmax(points.front().first,points.front().first);
		for(PPv::iterator i=points.begin();i!=points.end();++i)
		{
			minmax+=i->first;
		}
		Vector2<T> mid=(minmax.p1+minmax.p2)*0.5;
		Vector2<T> diag=minmax.Diagonal();
		T dmax=(diag.x>diag.y)?diag.x:diag.y;

		//Op`
		//Set up the supertriangle
		//This is a triangle which encompasses all the sample points.
		//The supertriangle coordinates are added to the end of the
		//vertex list. The supertriangle is the first triangle in
		//the triangle list.
		points.push_back(PP(Vector2<T>(mid.x-20*dmax,mid.y-dmax),0));
		points.push_back(PP(Vector2<T>(mid.x,mid.y+20*dmax),0));
		points.push_back(PP(Vector2<T>(mid.x+20*dmax,mid.y-dmax),0));

		EIv edges;
		std::vector<int> complete;
		complete.reserve(4*points.size());

		triangles.push_back(TriangleIndex(nv,nv+1,nv+2));
		complete.push_back(0);

		//Include each point one at a time into the existing mesh
		//bVɈ_ǉĂ
		for(int i=0;i<nv;++i)
		{
			/*if(dbg)
			{
				(*dbg).push_back(std::pair<TIv,PPv>(triangles,points));
			}*/

			const Vector2<T> &p=points[i].first;
			edges.clear();
//			edges.reserve(points.size());

			//Set up the edge buffer.
			//If the point (xp,yp) lies inside the circumcircle then the
			//three edges of that triangle are added to the edge buffer
			//and that triangle is removed.
			//GbWobt@쐬B
			//(xp,yp)Oډ~̓ɂ
			//Op`3̕ӂ̓GbWobt@ɒǉ
			//̎Op`͍폜
			for (int j=0;j<triangles.size();++j)
			{
				if (complete[j])
					continue;

				const Vector2<T> &p1=points[triangles[j].p1].first;
				const Vector2<T> &p2=points[triangles[j].p2].first;
				const Vector2<T> &p3=points[triangles[j].p3].first;

				Vector2<T> cr;
				T r;

				int inside = CircumCircle(p,p1,p2,p3,cr,r);
				/*int cross=0;
				if(CrossEdge(i,triangles[j].p1,triangles[j].p2,triangles[j].p3))
				{
					cross=1;
				}
				else if(SuperCrossEdge(i,triangles[j].p1,triangles[j].p2,triangles[j].p3,i))
				{
					//cross=2;
				}
				//int cross = CrossEdge(i,triangles[j].p1,triangles[j].p2,triangles[j].p3);
				if(inside)
				{
					if(UncrossEdge(i,triangles[j].p1,triangles[j].p2,triangles[j].p3))
					{
						inside=0;
					}
					else if(SuperUncrossEdge(i,triangles[j].p1,triangles[j].p2,triangles[j].p3,i))
					{
						inside=0;
					}
				}*/
				
				//if (inside||cross)
				if(inside)
				{
					//if(cross!=2)
					{
						edges.push_back(EdgeIndex(triangles[j].p1,triangles[j].p2));
						edges.push_back(EdgeIndex(triangles[j].p2,triangles[j].p3));
						edges.push_back(EdgeIndex(triangles[j].p3,triangles[j].p1));
					}

					triangles[j]=triangles.back();
					complete[j]=complete.back();

					triangles.pop_back();
					complete.pop_back();

					--j;
				}
			}//end of setting of the edge buffer.

			//Tag multiple edges
			//Note: if all triangles are specified anticlockwise then all
			//interior edges are opposite pointing in direction.
			//dӂɃ^O
			//FׂĂ̎Op`vɎw肳Ă
			//ׂĂ̕ӂ̒[_͋tɈʒu
			for (int j=0;j+1<edges.size();++j)
			{
				for (int k=j+1;k<edges.size();++k)
				{
					if ((edges[j].p1 == edges[k].p2) && (edges[j].p2 == edges[k].p1))
					{
						edges[j].p1 = -1;
						edges[j].p2 = -1;
						edges[k].p1 = -1;
						edges[k].p2 = -1;
					}
					//Shouldn't need the following, see note above
					//ʂɂ񂯂ǁAꉞ...
					/*if ((edges[j].p1 == edges[k].p1) && (edges[j].p2 == edges[k].p2))
					{
						edges[j].p1 = -1;
						edges[j].p2 = -1;
						edges[k].p1 = -1;
						edges[k].p2 = -1;
					}*/
				}
			}//end of tagging of multiple edges

			// Form new triangles for the current point
			// Skipping over any tagged edges.
			// All edges are arranged in clockwise order.
			//ړ_ɑ΂ĐVOp`\
			//^O̕tӂ̓XLbv
			//ׂĂ̕ӂ͎vɕׂ
			for (int j=0;j<edges.size();++j)
			{
				if (edges[j].p1 < 0 || edges[j].p2 < 0)
					continue;

				triangles.push_back(TriangleIndex(edges[j].p1,edges[j].p2,i));
				complete.push_back(0);
			}//end of forming of new triangles

		}//end of each point iteration

		//Remove triangles with supertriangle vertices
		//These are triangles which have a vertex number greater than nv
		//Op`(ԊÕ_~[Op`)̒_Op`菜
		//nv傫Ȓ_ԍ̒_Ă
		for (int i=0;i<triangles.size();++i)
		{
			if (triangles[i].p1 >= nv 
				|| triangles[i].p2 >= nv 
				|| triangles[i].p3 >= nv)
			{
				triangles[i]=triangles.back();
				triangles.pop_back();
				i--;
			}
		}

		//łɑOp`̒_菜
		for(int i=0;i<3;++i)
		{
			points.pop_back();
		}



		//
		/*for(int i=0;i<triangles.size();++i)
		{
			for(int j=0;j<connections.size();++j)
			{
				if( ( (triangles[i].p1==connections[j].p2)&&(triangles[i].p2==connections[j].p1) )||
					( (triangles[i].p2==connections[j].p2)&&(triangles[i].p3==connections[j].p1) )||
					( (triangles[i].p3==connections[j].p2)&&(triangles[i].p1==connections[j].p1) ) )
				{
					triangles[i]=triangles.back();
					triangles.pop_back();
					i--;
					break;
				}
			}
		}*/




		if(dbg)
		{
			(*dbg).push_back(DbgTris<T>(triangles,points,connections));
		}


		//ċAIɃGbWĂ
		for(int i=0;i<connections.size();++i)
		{
			edges.clear();
			EdgeIndex &c=connections[i];
			for(int j=0;j<triangles.size();++j)
			{
				TriangleIndex &t=triangles[j];
				if(CrossEdgeTriangle(c.p1,c.p2,t.p1,t.p2,t.p3))
				{
					edges.push_back(EdgeIndex(t.p1,t.p2));
					edges.push_back(EdgeIndex(t.p2,t.p3));
					edges.push_back(EdgeIndex(t.p3,t.p1));
					triangles[j]=triangles.back();//remove
					triangles.pop_back();
					--j;
				}
			}

			if(edges.empty())continue;

			//dGbW
			for (int j=0;j+1<edges.size();++j)
			{
				for (int k=j+1;k<edges.size();++k)
				{
					if ((edges[j].p1 == edges[k].p2) && (edges[j].p2 == edges[k].p1))
					{
						edges[j].p1 = -1;
						edges[j].p2 = -1;
						edges[k].p1 = -1;
						edges[k].p2 = -1;
					}
					//Shouldn't need the following, see note above
					//ʂɂ񂯂ǁAꉞ...
					if ((edges[j].p1 == edges[k].p1) && (edges[j].p2 == edges[k].p2))
					{
						edges[j].p1 = -1;
						edges[j].p2 = -1;
						edges[k].p1 = -1;
						edges[k].p2 = -1;
					}
				}
			}

			intv used(points.size());
			for(int j=0;j<edges.size();++j)
			{
				if(edges[j].p1==-1)//remove
				{
					edges[j]=edges.back();
					edges.pop_back();
					--j;					
				}
				else
				{
					used[edges[j].p1]=1;
					used[edges[j].p2]=1;
				}
			}

			Vector2<T> e0=points[c.p1].first,e1=points[c.p2].first;

			//GbW̍ƉE
			for(int k=0;k<2;++k)
			{
				//_W̃TuZbgAԍ
				TIv trianglesubset;
				PPv pointsubset;
				for(int j=0;j<points.size();++j)
				{
					if( ( used[j] && (IsLeft(e0,e1,points[j].first)^k) ) || j==c.p1 || j==c.p2 )
					{
						pointsubset.push_back(PP(points[j].first,j));
					}
				}
				if(pointsubset.size()<=2)
				{
					continue;
				}

				intv invmap(points.size(),-1);
				for(int j=0;j<pointsubset.size();++j)
				{
					invmap[pointsubset[j].second]=j;
				}
				EIv edgesubset;
				if(pointsubset.size()!=3)
				{
					for(int j=0;j<edges.size();++j)
					{
						int a=invmap[edges[j].p1];
						int b=invmap[edges[j].p2];
						if(a==-1 || b==-1)continue;
						edgesubset.push_back(EdgeIndex(a,b));
					}
				}
				Pv pointsubset2(pointsubset.size());
				for(int j=0;j<pointsubset.size();++j)
				{
					pointsubset2[j]=pointsubset[j].first;
				}
				DelauneyTriangulator<T>(pointsubset2,edgesubset,trianglesubset,dbg);

				//ԍɖ߂Đe̎Op`Wɒǉ
				for(int j=0;j<trianglesubset.size();++j)
				{
					triangles.push_back(TriangleIndex(
						pointsubset[trianglesubset[j].p1].second,
						pointsubset[trianglesubset[j].p2].second,
						pointsubset[trianglesubset[j].p3].second));
					/*trianglesubset[j].p1=pointsubset[trianglesubset[j].p1].second;
					trianglesubset[j].p2=pointsubset[trianglesubset[j].p2].second;
					trianglesubset[j].p3=pointsubset[trianglesubset[j].p3].second;
					triangles.push_back(trianglesubset[j]);*/
				}
			}


			if(dbg)
			{
				(*dbg).push_back(DbgTris<T>(triangles,points,connections));
			}
		}


		if(dbg)
		{
			(*dbg).push_back(DbgTris<T>(triangles,points,connections));
		}

		//O

		RemoveOutside();



		//
		for(int i=0;i<triangles.size();++i)
		{
			triangles[i].p1=points[triangles[i].p1].second;
			triangles[i].p2=points[triangles[i].p2].second;
			triangles[i].p3=points[triangles[i].p3].second;
		}


		if(dbg)
		{
			(*dbg).push_back(DbgTris<T>(triangles,points,connections));
		}

	}


private:

	/*void RemoveOutside()
	{
		if(connections.empty())return;

		intv outside(triangles.size(),0);
		EIv target;

		for(int i=0;i<connections.size();++i)
		{
			target.push_back(connections[i]);
			while(!target.empty())
			{
				EdgeIndex e=target.back();
				target.pop_back();

				//find next triangle
				for(int j=0;j<triangles.size();++j)
				{
					if(outside[j])continue;

					TriangleIndex &t=triangles[j];

					if( (e.p1==t.p1&&e.p2==t.p2)||
						(e.p1==t.p2&&e.p2==t.p3)||
						(e.p1==t.p3&&e.p2==t.p1) )
					{
						outside[j]=-1;
					}
					else if( e.p2==t.p1&&e.p1==t.p2 )
					{
						outside[j]=1;
						target.push_back(EdgeIndex(t.p2,t.p3));
						target.push_back(EdgeIndex(t.p3,t.p1));
					}
					else if( e.p2==t.p2&&e.p1==t.p3 )
					{
						outside[j]=1;
						target.push_back(EdgeIndex(t.p1,t.p2));
						target.push_back(EdgeIndex(t.p3,t.p1));
					}
					else if( e.p2==t.p3&&e.p1==t.p1 )
					{
						outside[j]=1;
						target.push_back(EdgeIndex(t.p1,t.p2));
						target.push_back(EdgeIndex(t.p2,t.p3));
					}
				}

				for(int j=0;j<target.size();++j)
				{
					for(int k=0;k<connections.size();++k)
					{
						//if(k==i)continue;

						if( (target[j].p1==connections[k].p1&&target[j].p2==connections[k].p2) ||
							(target[j].p2==connections[k].p1&&target[j].p1==connections[k].p2) )
						{
							target[j]=target.back();//remove
							target.pop_back();
							--j;
							break;
						}
					}
				}
			}
		}

		for(int i=0;i<outside.size();++i)
		{
			if(outside[i]==1)
			{
				triangles[i]=triangles.back();
				outside[i]=outside.back();
				triangles.pop_back();
				outside.pop_back();
				--i;
			}
		}	
	}*/


	void RemoveOutside()
	{
		if(connections.empty())return;

		intv outside(triangles.size(),0);

		for(int i=0;i<connections.size();++i)
		{
			EdgeIndex e=connections[i];
			intv targettriangle;

			for(int j=0;j<triangles.size();++j)
			{
				if(outside[j])continue;
				
				TriangleIndex &t=triangles[j];

				if( (e.p1==t.p1&&e.p2==t.p2)||
					(e.p1==t.p2&&e.p2==t.p3)||
					(e.p1==t.p3&&e.p2==t.p1) )
				{
					outside[j]=-1;
				}
				else if( (e.p2==t.p1&&e.p1==t.p2)||
					(e.p2==t.p2&&e.p1==t.p3)||
					(e.p2==t.p3&&e.p1==t.p1) )
				{
					outside[j]=1;
					targettriangle.push_back(j);
				}
			}

			while(!targettriangle.empty())
			{
				int tt=targettriangle.back();
				targettriangle.pop_back();

				TriangleIndex &e=triangles[tt];
				//find next triangle
				for(int j=0;j<triangles.size();++j)
				{
					if(outside[j])continue;

					TriangleIndex &t=triangles[j];

					if( (e.p1==t.p2&&e.p2==t.p1)||
						(e.p1==t.p3&&e.p2==t.p2)||
						(e.p1==t.p1&&e.p2==t.p3)||
						(e.p2==t.p2&&e.p3==t.p1)||
						(e.p2==t.p3&&e.p3==t.p2)||
						(e.p2==t.p1&&e.p3==t.p3)||
						(e.p3==t.p2&&e.p1==t.p1)||
						(e.p3==t.p3&&e.p1==t.p2)||
						(e.p3==t.p1&&e.p1==t.p3) )
					{
						//
						int crossconnection=0;
						for(int k=0;k<connections.size();++k)
						{
							if( (t.p1==connections[k].p1&&t.p2==connections[k].p2)||
								(t.p2==connections[k].p1&&t.p1==connections[k].p2)||
								(t.p2==connections[k].p1&&t.p3==connections[k].p2)||
								(t.p3==connections[k].p1&&t.p2==connections[k].p2)||
								(t.p3==connections[k].p1&&t.p1==connections[k].p2)||
								(t.p1==connections[k].p1&&t.p3==connections[k].p2) )
							{
								crossconnection=1;
								break;
							}
						}

						if(!crossconnection)
						{
							outside[j]=1;
							targettriangle.push_back(j);
						}
					}
				}
			}
		}

		for(int i=0;i<outside.size();++i)
		{
			if(outside[i]==1)
			{
				triangles[i]=triangles.back();
				outside[i]=outside.back();
				triangles.pop_back();
				outside.pop_back();
				--i;
			}
		}	
	}



	// Return TRUE if a point (xp,yp) is inside the circumcircle made up
	// of the points (x1,y1), (x2,y2), (x3,y3)
	// The circumcircle centre is returned in (xc,yc) and the radius r
	// NOTE: A point on the edge is inside the circumcircle
	// v1,v2,v3ȂOp`̊Oډ~̒pĂtrueԂ
	// Oډ~̒S_c,arꂼŏ
	// : GbW̓_Oډ~ɓĂ
	bool CircumCircle(const Vector2<T> &p,
		const Vector2<T> &v1,const Vector2<T> &v2,const Vector2<T> &v3,
		Vector2<T> &c,T &r)
	{
		double eps=1e-10;

		// Check for coincident points
		if (fabs(v1.y-v2.y) < eps && fabs(v2.y-v3.y) < eps)
		   return false;

		if (fabs(v2.y-v1.y) < eps)
		{
			double m2r = - (v3.x - v2.x) / (v3.y - v2.y);
			Vector2<T> m2 = (v2 + v3) * 0.5;
			c.x = (v2.x + v1.x) * 0.5;
			c.y = m2r * (c.x - m2.x) + m2.y;
		}
		else if (fabs(v3.y-v2.y) < eps)
		{
			double m1r = - (v2.x - v1.x) / (v2.y - v1.y);
			Vector2<T> m1 = (v1 + v2) * 0.5;
			c.x = (v3.x + v2.x) * 0.5;
			c.y = m1r * (c.x - m1.x) + m1.y;
		}
		else
		{
			double m1r = - (v2.x-v1.x) / (v2.y-v1.y);
			double m2r = - (v3.x-v2.x) / (v3.y-v2.y);
			Vector2<T> m1 = (v1 + v2) * 0.5;
			Vector2<T> m2 = (v2 + v3) * 0.5;
			c.x = (m1r * m1.x - m2r * m2.x + m2.y - m1.y) / (m1r - m2r);
			c.y = m1r * (c.x - m1.x) + m1.y;
		}

		Vector2<T> d1 = v2 - c;
		T sq1 = d1.Abs2();
		r = d1.Abs();

		Vector2<T> d2 = p - c;
		T sq2 = d2.Abs2();

		return (sq2<=sq1+eps)?true:false;

		// Suggested
		// return((drsqr <= rsqr + eps) ? TRUE : FALSE);
	}

	//(q1,q2)
	//Op`(p1,p2,p3)Ă
	//ڂĂ邾̏ꍇfalse
	bool CrossEdgeTriangle(int q1,int q2,int p1,int p2,int p3)
	{
		//ڂĂ邾
		if(q1==p1&&q2==p2)return false;
		if(q1==p2&&q2==p1)return false;
		if(q1==p2&&q2==p3)return false;
		if(q1==p3&&q2==p2)return false;
		if(q1==p3&&q2==p1)return false;
		if(q1==p1&&q2==p3)return false;
		
		const Vector2<T>
			&qq1 = points[q1].first,
			&qq2 = points[q2].first,
			&pp1 = points[p1].first,
			&pp2 = points[p2].first,
			&pp3 = points[p3].first;

		//1_L
		if(q1==p1||q2==p1)return CrossEdge0(qq1,qq2,pp2,pp3);
		if(q1==p2||q2==p2)return CrossEdge0(qq1,qq2,pp3,pp1);
		if(q1==p3||q2==p3)return CrossEdge0(qq1,qq2,pp1,pp2);

		return
			CrossEdge0(qq1,qq2,pp1,pp2)||
			CrossEdge0(qq1,qq2,pp2,pp3)||
			CrossEdge0(qq1,qq2,pp3,pp1);
	}
	
	//p00-p01ƕp10-p11̌
	bool CrossEdge0(const Vector2<T> &p00,const Vector2<T> &p01,
		const Vector2<T> &p10,const Vector2<T> &p11)
	{
		double eps=1e-10;

		Vector2<T>
			v00 = p00 - p10,
			v01 = p01 - p10,
			d = p11 - p10;

		T
			a = d.ExProd(v00),
			b = d.ExProd(v01);

		if(a<0 && b<0)
			return false;
		if(a>0 && b>0)
			return false;

		//s
		if(fabs(d.ExProd(v00-v01))<eps)
		{
			T
				l = d.Abs2(),
				s = d.DotProd(v00)/l,
				t = d.DotProd(v01)/l;
			if(0<=s && s<=1)return true;
			if(0<=t && t<=1)return true;
			if(t<0 && 1<s)return true;
			if(s<0 && 1<s)return true;

			return false;
		}
		else
		{
			T c=v01.ExProd(v00)/d.ExProd(v00-v01);

			if(c<0 || 1<c)
				return false;

			return true;
		}
	}

	int IsLeft(const Vector2<T> &e0,const Vector2<T> &e1,const Vector2<T> &p)
	{
		Vector2<T> v=e1-e0,d=p-e0;
		return (d.ExProd(v)<0)?1:0;
	}
	
	//for sort
	static int Compare(const void *v1,const void *v2)
	{
		PP *p1=(PP*)v1,*p2=(PP*)v2;
		if (p1->first.x < p2->first.x)
			return(-1);
		else if (p1->first.x > p2->first.x)
			return(1);
		else 
			return(0);
	}
};

};//end of namespace Rydot

#endif

