
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <math.h>
#include <matrix.h>
#include <matrixutil.h>

#include <ryaux.h>

#include "delauney.h"

#include "tetrahedron.h"

using namespace Rydot;

typedef std::vector<Vector2d> Vector2dv;

extern double pi;

//  2      3      2
//  +------+------+
//  |_    |_    |
//  |  _  |  _  |
//  |    _|    _|
//  +------+------+
//  0      1      0
int CheckVertex(const Vector2d &a,float rad)
{
	static const int tvi[]=
	{
		0,1,0,2,3,2,
	};
	static const double r3=sqrt(3.0)*0.5;

	Vector2d a2(a.x+a.y*0.5,a.y*r3);

	for(int i=0;i<3;i++)
	{
		for(int j=0;j<2;j++)
		{
			Vector2d ref(i+j*0.5,j*r3);
			if(a2.Dist(ref)<rad)
			{
				return tvi[j*3+i];
			}
		}
	}
	return -1;
}

//  +------+------+
//  |_    |_    |
//  | 0_1 | 2_3 |
//  |    _|    _|
//  +------+------+
// a is in [0:2][0:1]
int CheckFace(const Vector2d &a)
{
	if(a.x<1)
	{
		if(a.x+a.y<1)
		{
			return 0;
		}
		return 1;
	}
	if(a.x+a.y<2)
	{
		return 2;
	}
	return 3;
}

//ӏɂafc̒ɓ悤ɂ
//a[0:2][0:1]łƂB
Vector2d CorrectInFace(Vector2d a,int fc)
{
	Vector2d tmp=a;
	if(fc==0)
	{
		if(a.x+a.y>1)
		{
			a.x=2-a.x;
			if(a.x+a.y>1)return tmp;//Ȃ񂩂ւ
		}
		return a;
	}
	if(fc==1)
	{
		if(a.x>1)
		{
			a.x=2-a.x;
			if(a.x>1)return tmp;//Ȃ񂩂ւ
		}
		return a;
	}
	if(fc==2)
	{
		if(a.x<1)
		{
			a.x=2-a.x;
			if(a.x<1)return tmp;//Ȃ񂩂ւ
		}
		return a;
	}
	if(fc==3)
	{
		if(a.x+a.y<2)
		{
			a.x=2-a.x;
			if(a.x+a.y<2)return tmp;//Ȃ񂩂ւ
		}
		return a;
	}
	return a;
}

//     3      3
//  +------+------+
//  |_    |_    |
// 0| 4_ 1| 5_  |0
//  |    _|    _|
//  +------+------+
//     2      2

int CheckFaceCrossID(int af,int bf)
{
	static const int tcrs[]=
	{
		-1, 4, 2, 0,
		 4,-1, 1, 3,
		 2, 1,-1, 5,
		 0, 3, 5,-1,
	};

	return tcrs[af*4+bf];
}

// not crossed: -1
// c,d͌_0,2,3
//  aɋ߂قc,
//  bɋ߂قd
int CheckFaceCross(const Vector2d &a,const Vector2d &b,Vector2d &c,Vector2d &d)
{
	int af=CheckFace(a);
	int bf=CheckFace(b);

	int crs=CheckFaceCrossID(af,bf);
	
	if(crs==-1)return crs;
	
	if(crs==0)
	{
		if(af==0){ReflectLineRay(a,b-Vector2d(2,0),Rect2d(0,1,0,0),c);d=c+Vector2d(2,0);}//af:0 bf:3
		else{ReflectLineRay(a-Vector2d(2,0),b,Rect2d(0,0,0,1),d);c=d+Vector2d(2,0);}//af:3 bf:0
		return crs;
	}
	if(crs==1)
	{
		if(af==1)ReflectLineRay(a,b,Rect2d(1,0,1,1),c);//af:1 bf:2
		else ReflectLineRay(a,b,Rect2d(1,1,1,0),c);//af:2 bf:1
		d=c;
		return crs;
	}
	if(crs==2)
	{
		if(af==0){ReflectLineRay(a,Vector2d(2,0)-b,Rect2d(0,0,1,0),c);d=Vector2d(2,0)-c;}//af:0 bf:2
		else{ReflectLineRay(Vector2d(2,0)-a,b,Rect2d(1,0,0,0),d);c=Vector2d(2,0)-d;}//af:2 bf:0
		return crs;
	}
	if(crs==3)
	{
		if(af==1){ReflectLineRay(a,Vector2d(2,2)-b,Rect2d(1,1,0,1),c);d=Vector2d(2,2)-c;}//af:1 bf:3
		else{ReflectLineRay(Vector2d(2,2)-a,b,Rect2d(0,1,1,1),d);c=Vector2d(2,2)-d;}//af:3 bf:1
		return crs;
	}
	if(crs==4)
	{
		if(af==0)ReflectLineRay(a,b,Rect2d(1,0,0,1),c);//af:0 bf:1
		else ReflectLineRay(a,b,Rect2d(0,1,1,0),c);//af:1 bf:0
		d=c;
		return crs;
	}
	if(crs==5)
	{
		if(af==2)ReflectLineRay(a,b,Rect2d(2,0,1,1),c);//af:2 bf:3
		else ReflectLineRay(a,b,Rect2d(1,1,2,0),c);//af:3 bf:2
		d=c;
		return crs;
	}
	return -1;
}

int CheckOnEdge(const Vector2d &a,const Vector2d &b)
{
	Vector2d root[]={
		Vector2d(0,0),//vertical
		Vector2d(1,0),
		Vector2d(2,0),

		Vector2d(0,0),//horizontal
		Vector2d(1,0),
		Vector2d(0,1),
		Vector2d(1,0),

		Vector2d(1,0),//diagonal
		Vector2d(2,0),
	};
	Vector2d dir[]={
		Vector2d(0,1),
		Vector2d(0,1),
		Vector2d(0,1),

		Vector2d(1,0),
		Vector2d(1,0),
		Vector2d(1,0),
		Vector2d(1,0),

		Vector2d(-1,1),
		Vector2d(-1,1),
	};

	double eps=1e-10;

	for(int i=0;i<9;i++)
	{
		Vector2d da=a-root[i];
		if(fabs(da.ExProd(dir[i]))>eps)continue;

		Vector2d db=b-root[i];
		if(fabs(db.ExProd(dir[i]))>eps)continue;

		Vector2d dab=b-a;
		if(fabs(dab.ExProd(dir[i]))>eps)continue;

		return 1;
	}
	return 0;
}


template<class Container,class Itrator>
std::pair<Container,Container> split(Container &c,Itrator i)
{
	return std::pair<Container,Container>(Container(c.begin(),i),Container(i,c.end()));
}



///////////////////////////////////////////////////////////
// class Edge
//

int Edge::CheckSelfEdgeCross(const Vector2d &a,const Vector2d &b)
{
	for(int i=0;i<pedges.size();i++)
	{
		Vector2d cf;

		if(pedges[i].face==-1)continue;

		if(a==points[pedges[i].a]||a==points[pedges[i].b])continue;
		if(b==points[pedges[i].a]||b==points[pedges[i].b])continue;

		Vector2d r1=points[pedges[i].a],r2=points[pedges[i].b];
		if(r1==r2)continue;

		if(ReflectLineRay(a,b,Rect2d(r1,r2),cf))
		{
			return 1;
		}
		else if(ReflectLineRay(a,b,Rect2d(r2,r1),cf))
		{
			return 1;
		}	
	}
	return 0;
}

//̕ӂĂ̔IDԂ
int Edge::CheckCross(const Vector2d &a,const Vector2d &b,Vector2d &c)
{
	double eps=1e-3;
	double min=1e5;
	int minid=-1;
	Vector2d minc;
	for(int i=0;i<pedges.size();i++)
	{
		Vector2d cf;

		if(pedges[i].face==-1)continue;

		if(ReflectLineRay(a,b,Rect2d(points[pedges[i].a],points[pedges[i].b]),cf))
		{
			//conflict
			double d=cf.Dist(a);

			if(CheckVertex(a,eps)!=-1)continue;

			if(d<min)
			{
				min=d;
				minid=i;
				minc=cf;
			}
		}
		else if(ReflectLineRay(a,b,Rect2d(points[pedges[i].b],points[pedges[i].a]),cf))
		{
			//conflict
			double d=cf.Dist(a);

			if(CheckVertex(a,eps)!=-1)continue;

			if(d<min)
			{
				min=d;
				minid=i;
				minc=cf;
			}
		}
	}
	c=minc;
	return minid;
}

int Edge::Append(const Vector2d &a,int fc)
{
	points.push_back(a);
	int aa=points.size();
	if(aa>=2)
	{
		pedges.push_back(PartialEdge(aa-2,aa-1,fc));
	}
	return 1;
}

// se:split edge
// c:split point
std::vector<Edge> Edge::Split(int se,const Vector2d &sp)
{
	std::vector<Edge> ev;
	if(se<0)return ev;
	
	ev.resize(2);
	
	int aa=pedges[se].b;
	
	std::pair<Vector2dv,Vector2dv> pspl=split(points,points.begin()+aa);
	std::pair<pev,pev> espl=split(pedges,pedges.begin()+se);
	
	ev[0].points=pspl.first;
	ev[0].points.push_back(sp);
	int bb=ev[0].points.size()-1;
	int fc=pedges[se].face;
	ev[0].pedges=espl.first;
	ev[0].pedges.push_back(PartialEdge(bb-1,bb,fc));
	
	ev[1].points.push_back(sp);
	ev[1].points.insert(ev[1].points.end(),pspl.second.begin(),pspl.second.end());
	for(int i=0;i<espl.second.size();i++)
	{
		ev[1].pedges.push_back(PartialEdge(i,i+1,espl.second[i].face));
	}

	return ev;
}

void Edge::SplitByFace(std::vector< std::vector<Edge> > &r,std::vector< std::pair<Vector2d,int> > &middlevertex)
{
	Edge tmp;

	//dbgout
	std::cout<<"SplitByFace"<<std::endl;
	for(int i=0;i<pedges.size();++i)
	{
		std::cout<<pedges[i].face<<" ";
	}
	std::cout<<std::endl;

	int f=pedges[0].face;
	tmp.Append(points[0],f);
	tmp.r1=r1;
	tmp.id=0;
	int lastf=-1;
	int lastr1=-1;
	for(int i=0;i<pedges.size();++i)
	{
		if(f==-1)
		{
			f=pedges[i+1].face;
			continue;
		}

		if(pedges[i].face==-1)
		{
			/*if(f==-1)
			{
				f=pedges[i+1].face;
				continue;
			}*/

			tmp.Append(points[i],f);
			tmp.r2=middlevertex.size();

			//_Ȃׂǂ
			int simplex=1;
			for(int j=0;j+1<tmp.points.size();++j)
			{
				if(tmp.points[j]!=tmp.points[j+1])
				{
					r[f].push_back(tmp);
					tmp.Clear();
					tmp.r1=middlevertex.size();
					int cr=CheckFaceCrossID(f,pedges[i+1].face);
					middlevertex.push_back( std::pair<Vector2d,int>(points[i],cr) );
					lastf=f;
					lastr1=tmp.r1;
					f=pedges[i+1].face;
					simplex=0;


					//dbg
					std::cout<<"append:"<<lastf<<" to:"<<f<<std::endl;


					break;
				}
			}

			if(simplex)
			{
				
				//dbg
				std::cout<<"simplex:"<<f<<std::endl;



				tmp.Clear();
				f=pedges[i+1].face;
			}
			
			continue;
		}
		tmp.Append(points[i],f);
	}
	tmp.Append(points.back(),f);
	tmp.r2=r2;

	//_Ȃׂǂ
	for(int i=0;i+1<tmp.points.size();++i)
	{
		if(tmp.points[i]!=tmp.points[i+1])
		{
			
			//dbg
			std::cout<<"lastappend:"<<f<<std::endl;


			r[f].push_back(tmp);
			return;
		}
	}

	if(lastf>=0)
	{

		//dbg
		std::cout<<"lastsimplex:"<<f<<std::endl;

		r[lastf].back().r2=r2;
		middlevertex.pop_back();
	}

	return;
}




///////////////////////////////////////////////////////////
//


///////////////////////////////////////////////////////////
// class TransformTree
//
//WJAj邽߂̕ϊp̖
//Op`̒Sl
//eW
//->rotate_alpha(eW炻̎Op`ɂ)
//->offset_radius(Op`̓ډ~̔a+xɕsړ)
//->rotate_phi(ʊp)
//->offset_radius(Op`̓ډ~̔a+xɕsړ)
//->rotate_beta(ŎqWnɂȂ)
//->offset_center(phaseWOp`̌_)
//->shear(phaseWOp`)
//->offset_phase(phaseW_Ɉړ)
//΂͂^^;

//ʂʂւ킽鎞̍Wϊ
void TransformTree::SetFace(int af,int bf)
{
	const double pi1=pi/3.0,pi2=pi*2.0/3.0,pi3=pi,pi4=pi*4.0/3.0,pi5=pi*5.0/3.0;

	static const double dir[]=
	{
		  0,pi1,pi3,pi5,
		pi3,  0,pi1,pi5,
		pi3,pi5,  0,pi1,
		pi1,pi5,pi3,  0
	};

	static const double rot[]=
	{
		  0,  0,  0,pi2,
		pi2,  0,pi4,pi4,
		  0,pi2,  0,  0,
		pi4,pi4,pi2,  0
	};

	static const Vector2d ofs_phase[4]=
	{
		Vector2d(0,0),
		Vector2d(0,-1),
		Vector2d(-1,0),
		Vector2d(-1,-1)
	};

	static const Matrix22d
		shear1(
			1,0,
			1,1),
		shear2(
			1,-1,
			0,1),
		shear3(
			1,0.5,
			0,sqrt(3.0)*0.5),
		shear4=shear3*shear2*shear1;

	static const Matrix22d shear_phase[4]=
	{
		shear3,
		shear4,
		shear3,
		shear4
	};

	rotate_alpha=dir[af*4+bf];
	offset_radius=sqrt(3.0)/6.0;
	rotate_beta=rot[af*4+bf];
	offset_center=Vector2d(-0.5,-sqrt(3.0)/6.0);
	shear=shear_phase[bf];
	offset_phase=ofs_phase[bf];
}






///////////////////////////////////////////////////////////
// class TekitohPhase
//
// lʑ̂eLg[ɕό`Ĉ
//

int TekitohPhase::SaveStroke(std::string fn)
{
	FILE *fp=fopen(fn.c_str(),"wb");

	if(fp==NULL)return 0;

	int k=validedges.size();
	fwrite(&k,sizeof(int),1,fp);

	for(int i=0;i<validedges.size();++i)
	{
		int k=validedges[i].points.size();
		fwrite(&k,sizeof(int),1,fp);

		for(int j=0;j<validedges[i].points.size();++j)
		{
			fwrite(&(validedges[i].points[j]),sizeof(float),2,fp);
		}

		k=validedges[i].pedges.size();
		fwrite(&k,sizeof(int),1,fp);

		for(int j=0;j<validedges[i].pedges.size();++j)
		{
			fwrite(&(validedges[i].pedges[j]),sizeof(int),3,fp);
		}

		fwrite(&(validedges[i].r1),sizeof(int),1,fp);
		fwrite(&(validedges[i].r2),sizeof(int),1,fp);
	}

	fwrite(vertexused,sizeof(int),6,fp);

	fwrite(&externalcrossnumber,sizeof(int),1,fp);

	fwrite(crossed,sizeof(float),4,fp);

	fclose(fp);

	return 1;
}

int TekitohPhase::LoadStroke(std::string fn)
{
	FILE *fp=fopen(fn.c_str(),"rb");

	if(fp==NULL)return 0;

	int k;

	fread(&k,sizeof(int),1,fp);

	validedges.clear();
	validedges.resize(k);

	for(int i=0;i<validedges.size();++i)
	{
		fread(&k,sizeof(int),1,fp);

		validedges[i].points.clear();
		validedges[i].points.resize(k);

		for(int j=0;j<validedges[i].points.size();++j)
		{
			fread(&validedges[i].points[j],sizeof(float),2,fp);
		}

		fread(&k,sizeof(int),1,fp);
		validedges[i].pedges.clear();
		validedges[i].pedges.resize(k);

		for(int j=0;j<validedges[i].pedges.size();++j)
		{
			fread(&validedges[i].pedges[j],sizeof(int),3,fp);
		}

		fread(&validedges[i].r1,sizeof(int),1,fp);
		fread(&validedges[i].r2,sizeof(int),1,fp);
	}

	fread(vertexused,sizeof(int),6,fp);

	fread(&externalcrossnumber,sizeof(int),1,fp);

	fread(crossed,sizeof(float),4,fp);

	fclose(fp);

	return 1;
}

int TekitohPhase::SaveDelauney(std::string fn,int n)
{
	FILE *fp=fopen(fn.c_str(),"wt");

	if(fp==NULL)return 0;

	fprintf(fp,"%d\n",recpolygons[n].p.size());
	for(int i=0;i<recpolygons[n].p.size();++i)
	{
		fprintf(fp,"%.40lf\t%.40lf\n",recpolygons[n].p[i].x,recpolygons[n].p[i].y);
	}

	fclose(fp);

	return 1;
}

int TekitohPhase::Startable(const Vector2d &a)
{
	if(!presentedge.Empty())return -1;

	double rad=0.1;
	int v=CheckVertex(a,rad);

	if(CheckFinish())return -1;

	return v;
}

bool TekitohPhase::EndableVertex(int v)
{
	if(presentedge.Empty())return false;

	int s=CheckVertex(presentedge.GetStart(),0.1);

	if(v==s)return false;

	if(CheckLink(s,v))return false;

	return true;
}

bool TekitohPhase::EndableEdge(int e)
{
	if(presentedge.Empty())return false;

	int s=CheckVertex(presentedge.GetStart(),0.1);

	if(CheckLoop(e,s))return false;

	return true;
}

void TekitohPhase::Clear()
{
	presentedge.Clear();

	for(int i=0;i<6;++i)
	{
		vertexused[i]=0;
	}

	externalcrossnumber=0;
	
	validedges.clear();

//		loop.clear();
	loopface.clear();
	mid.clear();
	splitededges.clear();
	recpolygons.clear();
	tts.clear();
	region=Rect2i(0,0,0,0);

	crossed[0]=Vector2d(-1,-1);
	crossed[1]=Vector2d(-1,-1);
}

int TekitohPhase::CalcVertexUsed()
{
	for(int i=0;i<6;++i)
	{
		vertexused[i]=0;
	}
	for(int i=0;i<validedges.size();++i)
	{
		int a=validedges[i].r1,b=validedges[i].r2;

		vertexused[a]++;
		vertexused[b]++;

		if(a>=4)
		{
			crossed[a-4]=validedges[i].GetStart();
		}
		if(b>=4)
		{
			crossed[b-4]=validedges[i].GetPresent();
		}
	}
	return 1;
}

int TekitohPhase::CheckFinish()
{
	if(validedges.size()<3)return 0;
	intv checked(4+externalcrossnumber);
	int a=validedges[0].r1,b=validedges[0].r2;
	checked[a]=1;
	checked[b]=1;
	int app=1;
	while(app)
	{
		app=0;

		for(int i=0;i<validedges.size();i++)
		{
			a=validedges[i].r1,b=validedges[i].r2;
			if(checked[a]&&(!checked[b]))
			{
				checked[b]=1;
				app=1;
			}
			if(checked[b]&&(!checked[a]))
			{
				checked[a]=1;
				app=1;
			}
		}
	}
	for(int i=0;i<checked.size();++i)
	{
		if(!checked[i])
			return 0;
	}
	return 1;
}

int TekitohPhase::CheckLink(int sv,int dest)
{
	if(sv<0)return 0;
	if(dest<0)return 0;
	intv checked(6);
	checked[sv]=1;
	for(int i=0;i<validedges.size();++i)
	{
		int a=validedges[i].r1,b=validedges[i].r2;
		if(a!=sv&&b!=sv)
		{
			continue;
		}

		int app=1;
		while(app)
		{
			if(checked[dest])
				return 1;

			app=0;

			for(int i=0;i<validedges.size();i++)
			{
				a=validedges[i].r1,b=validedges[i].r2;
				if(checked[a]&&(!checked[b]))
				{
					checked[b]=1;
					app=1;
				}
				if(checked[b]&&(!checked[a]))
				{
					checked[a]=1;
					app=1;
				}
			}
		}
	}
	return 0;
}

//validedges[ne]ɂȂĂvalidedgeŝȂ
//svɂȂĂ̂1([v)
int TekitohPhase::CheckLoop(int ne,int sv)
{
	if(sv<0)return 0;
	if(ne<0)return 0;

	intv checked(6);
	int a=validedges[ne].r1,b=validedges[ne].r2;
	checked[a]=1;
	checked[b]=1;
	int app=1;
	while(app)
	{
		if(checked[sv])
			return 1;

		app=0;

		for(int i=0;i<validedges.size();i++)
		{
			a=validedges[i].r1,b=validedges[i].r2;
			if(checked[a]&&(!checked[b]))
			{
				checked[b]=1;
				app=1;
			}
			if(checked[b]&&(!checked[a]))
			{
				checked[a]=1;
				app=1;
			}
		}
	}
	return 0;
}

int TekitohPhase::CheckValidEdgeCross(const Vector2d &a,const Vector2d &b,Vector2d &c,int &ne)
{
	double min=1e5;
	int minid=-1;
	Vector2d minc;
	ne=-1;
	for(int i=0;i<validedges.size();++i)
	{
		Vector2d cf;
		int ci=validedges[i].CheckCross(a,b,cf);
		
		double d=cf.Dist(a);
		
		if(d<min)
		{
			min=d;
			minid=ci;
			minc=cf;
			ne=i;
		}
	}
	c=minc;
	return minid;
}

//returns
//0:cannot append
//1:append succeed
//2:crossed
int TekitohPhase::Append(const Vector2d &a)
{
	if(!presentedge.Empty())
	{
		Vector2d c1,c2;
		const Vector2d &present=presentedge.GetPresent();
		//ʂn邩ǂ
		int cf=CheckFaceCross(present,a,c1,c2);
		if(cf!=-1)//ʂn
		{
			//_̏Ԃ
			//present(ݒn),c3(),c1(ʂ̒[)
			//ʂn
			//c2(ׂ̖ʂ̒[),c4(),a(s)
			Vector2d c3;
			int ne1;
			int ce1=CheckValidEdgeCross(present,c1,c3,ne1);
			
			if(ce1!=-1)//OŌ
			{
				//Ȍ
				if(presentedge.CheckSelfEdgeCross(present,c3))
				{
					//presentedge.Clear();
					return 0;
				}
				
				//presentedge.Append(c3,CheckFace(present));

				if(ApplyCross(ne1,ce1,c3,CheckFace(present)))
				{
					return 2;
				}
				else
				{
					return 0;
				}
			}
			else
			{
				//Ȍ
				if(presentedge.CheckSelfEdgeCross(present,c1))
				{
					//presentedge.Clear();
					return 0;
				}
			}

			//ʂn

			Edge tmp=presentedge;

			presentedge.Append(c1,CheckFace(present));
			presentedge.Append(c2,-1);
			
			Vector2d c4;
			int ne2;
			int ce2=CheckValidEdgeCross(c2,a,c4,ne2);
			
			if(ce2!=-1)//ʂnŌ
			{
				//Ȍ
				if(presentedge.CheckSelfEdgeCross(c2,c4))
				{
					presentedge=tmp;
					//presentedge.Clear();
					return 0;
				}

				//presentedge.Append(c1,CheckFace(present));
				//presentedge.Append(c2,-1);//c1-c2Ԃ͖ʂɑȂB

				//presentedge.Append(c4,CheckFace(a));
				
				if(ApplyCross(ne2,ce2,c4,CheckFace(a)))
				{
					return 2;
				}
				else
				{
					presentedge=tmp;
					return 0;
				}	
			}
			else
			{
				//Ȍ
				if(presentedge.CheckSelfEdgeCross(c2,a))
				{
					presentedge=tmp;
					//presentedge.Clear();
					return 0;
				}
			}

			//presentedge.Append(c1,CheckFace(present));
			//presentedge.Append(c2,-1);
			
			//Ȃ
			presentedge.Append(a,CheckFace(a));
			return 1;
		}
		else//ʂnȂ
		{
			Vector2d c3;
			int ne;
			int ce=CheckValidEdgeCross(present,a,c3,ne);
			
			if(ce!=-1)//
			{
				//Ȍ
				if(presentedge.CheckSelfEdgeCross(present,c3))
				{
					//presentedge.Clear();
					return 0;
				}
				
				//presentedge.Append(c3,CheckFace(a));

				if(ApplyCross(ne,ce,c3,CheckFace(a)))
				{
					return 2;
				}
				else
				{
					return 0;
				}
			}
			else
			{
				//Ȍ
				if(presentedge.CheckSelfEdgeCross(present,a))
				{
					//presentedge.Clear();
					return 0;
				}
			}

			//Ȃ
			double eps=0.02;

			double d=a.Dist(present);
			if(d<eps)//too short
			{
				return 1;
			}
			
			presentedge.Append(a,CheckFace(a));
			return 1;
		}
	}
	else
	{
		double rad=0.1;
		int v=CheckVertex(a,rad);

		if(CheckFinish())return 0;

		if(v!=-1)
		{
			Vector2d a2(v%2,v/2);
			presentedge.Start(a2);
			return 1;
		}
		return 0;
	}
}

int TekitohPhase::Apply()
{
	if(!presentedge.Empty())
	{
		Vector2d present=presentedge.GetPresent();
		Vector2d start=presentedge.GetStart();
		double rad=0.1;
		int v=CheckVertex(present,rad);

		int sv=CheckVertex(start,rad);

		if(v<0)//pŏIĂȂ
		{
			return 0;
		}

		if(v==sv)//Jn_ƂȂ
		{
			//presentedge.Clear();
			return 0;
		}

		if(CheckLink(sv,v))//[vĂ
		{
			//presentedge.Clear();
			return 0;
		}

		if(v!=-1)
		{
			Vector2d a2(v%2,v/2);

			int ra=Append(a2);
			if(ra==2)//NXŏI
			{
				return 2;
			}
			else if(ra==1)
			{

				//ʂɒ_ŏI

				presentedge.Validate(sv,v);
				
				validedges.push_back(presentedge);
				
				presentedge.Clear();

				CalcVertexUsed();

				return 1;
			}
			presentedge.Clear();
			return 0;
		}
	}
	return 0;
}

//ne:crossed valid edge
//ce:crossed partialedge
//c:cross point
int TekitohPhase::ApplyCross(int ne,int ce,const Vector2d &c,int f)
{
	if(externalcrossnumber>=2)
	{
		Failure();
		return 0;
	}

	double rad=0.1;
	int sv=CheckVertex(presentedge.GetStart(),rad);

	int lp=CheckLoop(ne,sv);

	if(lp)
	{
		//Failure();
		return 0;
	}

	presentedge.Append(c,f);

	ev espl=validedges[ne].Split(ce,c);

	int a=validedges[ne].r1,b=validedges[ne].r2;
	int cc=4+externalcrossnumber;

	espl[0].Validate(a,cc);
	espl[1].Validate(cc,b);
	presentedge.Validate(sv,cc);
	validedges[ne]=espl[0];
	validedges.push_back(espl[1]);
	validedges.push_back(presentedge);
	presentedge.Clear();

	externalcrossnumber++;

	CalcVertexUsed();

	return 1;
}

int TekitohPhase::SplitByFace()
{
	if(!CheckFinish())return 0;

	mid.clear();
	mid.resize(6);

	//_ƌ_
	Vector2d vtx[]=
	{
		Vector2d(0,0),
		Vector2d(1,0),
		Vector2d(0,1),
		Vector2d(1,1),
		crossed[0],
		crossed[1],
	};

	//mid:WJɕ}`̒_
	//Ɏlʑ̂̒_Ɛؒf̌_Ă
	for(int i=0;i<6;i++)
	{
		mid[i].second=-1;
		mid[i].first=vtx[i];
	}

	//splitedges:WJɕ}`̕ӂ̎lʑ̖̂ʂŕނ
	splitededges.resize(4);

	for(int i=0;i<validedges.size();++i)
	{
		validedges[i].SplitByFace(splitededges,mid);
	}

	int sz=mid.size();

	//sort
	//[[lʑ̂̒_]ƌ_]ȊO̒_eӂƂɃ\[g
	std::vector< std::vector<int> > sortmid(6);

	for(int i=0;i<mid.size();i++)
	{
		int e=mid[i].second;
		if(e>=0)
			sortmid[e].push_back(i);
	}

	//\[g͂Ȃ
	//     3   3
	//  +------+------+
	//  |_    |_    |
	// 0| 4_ 1| 5_  |0
	//  |  _|  _|
	//  +------+------+
	//     2   2
	Vector2d p(1,-1);

	for(int i=0;i<6;++i)
	{
		for(int j=0;j+1<sortmid[i].size();++j)
		{
			for(int k=j+1;k<sortmid[i].size();++k)
			{
				if( mid[sortmid[i][k]].first.Dist2(p)<mid[sortmid[i][j]].first.Dist2(p) )
				{
					std::swap(sortmid[i][k],sortmid[i][j]);
				}
			}
		}
	}


	std::cout<<"sorted midpoint"<<std::endl;
	for(int i=0;i<6;++i)
	{
		std::cout<<"edge:"<<i<<std::endl;
		for(int j=0;j<sortmid[i].size();++j)
		{
			std::cout<<"("<<mid[sortmid[i][j]].first<<")";
		}
		std::cout<<std::endl;
	}




	//LŃ\[ĝ̊eӂł̕
	//vertex,edge, direction,...
	int fe[]=
	{
		0,2,-1, 1,4,1, 2,0,-1,//face0
		2,4,-1, 1,1,1, 3,3,1,
		3,1,-1, 1,2,1, 0,5,1,
		3,5,-1, 0,0,1, 2,3,-1,//face3
	};

	//eʂ̂ӂ̕𐶐Asplitedgesɒǉ
	for(int f=0;f<4;++f)
	{
		//̕
		std::vector<int> fm;
		for(int i=0;i<3;++i)
		{
			fm.push_back(fe[f*9+i*3]);
			int e=fe[f*9+i*3+1];
			int dir=fe[f*9+i*3+2];
			int smsz=sortmid[e].size();
			for(int j=0;j<smsz;++j)
			{
				if(dir==1)
				{
					fm.push_back(sortmid[e][j]);
				}
				else
				{
					fm.push_back(sortmid[e][smsz-j-1]);
				}
			}
		}
		int fmsz=fm.size();
		for(int i=0;i<fmsz;++i)
		{
			//mat[ fm[i]*sz+fm[(i+1)%fmsz] ]++;

			//midpointfɓ悤ɂ
			Vector2d p1=CorrectInFace(mid[fm[i]].first,f);
			Vector2d p2=CorrectInFace(mid[ fm[(i+1)%fmsz] ].first,f);

			Edge e;
			e.Append(p1,f);
			e.Append(p2,f);
			//e.Append(mid[fm[i]].first,f);
			//e.Append(mid[ fm[(i+1)%fmsz] ].first,f);
			e.r1=fm[i];
			e.r2=fm[(i+1)%fmsz];
			e.id=1;
			splitededges[f].push_back(e);
		}
	}

	std::cout<<"splitedges"<<std::endl;
	for(int i=0;i<4;++i)
	{
		std::cout<<"f:"<<i<<std::endl;
		for(int j=0;j<splitededges[i].size();++j)
		{
			std::cout<<splitededges[i][j].points<<std::endl;
		}
	}

	//[GbW̔wʂɑΉGbWN
	//\@ backlink = [ʔԍ] + [̖ʂsplitededgeԍ]*4
	for(int i=0;i+1<4;++i)
	{
		for(int j=0;j<splitededges[i].size();++j)
		{
			if(splitededges[i][j].id!=1)continue;
			for(int k=i+1;k<4;k++)
			{
				for(int n=0;n<splitededges[k].size();++n)
				{
					if(splitededges[k][n].id!=1)continue;

					if(splitededges[i][j].r1==splitededges[k][n].r2 &&
						splitededges[i][j].r2==splitededges[k][n].r1)
					{
						splitededges[i][j].backlink=n*4+k;
						splitededges[k][n].backlink=j*4+i;
					}
				}
			}
		}
	}

	loopface.clear();

	for(int f=0;f<4;++f)
	{
		ExtractLoop(f);
	}

	//eʂƂɃ[v
/*		for(int f=0;f<4;++f)
	{
		//אڍs
		std::vector<int> mat;
		mat.resize(sz*sz);

		//Xg[N𕪊
		for(int i=0;i<splitededges[f].size();++i)
		{
			int a=splitededges[f][i].r1,b=splitededges[f][i].r2;
			if(splitededges[f][i].id==1)
			{
				mat[a*sz+b]++;
			}
			else
			{
				mat[a*sz+b]++;
				mat[b*sz+a]++;
			}
		}

		std::cout<<"---"<<std::endl;

		while(1)
		{
			int r=ExtractLoop(mat,sz,loop);
			if(r!=1)break;
			else
				loopface.push_back(f);
		}
	}

	std::cout<<"loop.size:"<<loop.size()<<std::endl;

	for(int i=0;i<loop.size();++i)
	{
		for(int j=0;j<loop[i].size();++j)
		{
			int a=loop[i][j]/sz;
			int b=loop[i][j]%sz;
			std::cout<<"("<<a<<","<<b<<") ";
		}
		std::cout<<std::endl;
	}*/

	MakePolygon();

	OffsetPolygon();

	return 1;
}

int TekitohPhase::ExtractLoop(int f)
{
	std::vector< LoopEdge > loopedge;
	std::vector< LoopVertex > loopvertex;

	loopedge.clear();
	loopvertex.clear();

	for(int i=0;i<splitededges[f].size();++i)
	{
		loopedge.push_back(LoopEdge(splitededges[f][i].r1,splitededges[f][i].r2,i,splitededges[f][i].id));
	}

	loopvertex.resize(mid.size());

	for(int i=0;i<loopedge.size();++i)
	{
		loopvertex[loopedge[i].r1].v=loopedge[i].r1;
		loopvertex[loopedge[i].r1].e.push_back(i);
		loopvertex[loopedge[i].r2].v=loopedge[i].r2;
		loopvertex[loopedge[i].r2].e.push_back(i);
	}

	ExtractLoopSub(loopedge,loopvertex,loopface,f);

	return 1;
}


int TekitohPhase::ExtractLoopSub(std::vector<LoopEdge> &e,std::vector<LoopVertex> &v,std::vector<LoopVertex> &fout,int f)
{
	//find 2 dir
	int found=-1;
	for(int i=0;i<e.size();++i)
	{
		if(e[i].id!=1)
		{
			found=i;
			break;
		}
	}

	if(found==-1)//ŏ[vۂ
	{
		LoopVertex fc;
		fc.v=f;
		for(int i=0;i<e.size();++i)
		{
			fc.e.push_back(e[i].e);
		}
		fout.push_back(fc);
		return 0;
	}

	LoopEdge &fe=e[found];

	//̂΂
	intv checkededge(e.size(),0);

	checkededge[found]=1;

	intv pathr1;
	StretchEdge(fe.r1,e,v,checkededge,pathr1);
	intv pathr2;
	StretchEdge(fe.r2,e,v,checkededge,pathr2);

	intv mergepath;

	for(int i=1;i<pathr1.size();++i)
	{
		mergepath.push_back(pathr1[i]);
	}
	mergepath.push_back(found);
	for(int i=pathr2.size();i>1;--i)
	{
		mergepath.push_back(pathr2[i-1]);
	}

	if(pathr1.empty()||pathr2.empty())
	{
		//error
		std::cout<<"error: could not stretch edge!"<<std::endl;
		return -1;
	}

	//͂`FbN
	LoopEdge &e1=e[pathr1[0]];
	LoopEdge &e2=e[pathr2[0]];

	int r1;
	if(e1.r1==e[mergepath.front()].r1||e1.r1==e[mergepath.front()].r2)r1=e1.r1;
	else r1=e1.r2;

	int r2;
	if(e2.r2==e[mergepath.back()].r1||e2.r2==e[mergepath.back()].r2)r2=e2.r2;
	else r2=e2.r1;

	intv p;
	intv q;
	if(v[r1].e.size()==3)
	{
		for(int i=0;i<3;++i)
		{
			if(v[r1].e[i]!=mergepath.front())
			{
				q.push_back(v[r1].e[i]);
				if(e[v[r1].e[i]].r1!=r1)p.push_back(e[v[r1].e[i]].r1);
				else p.push_back(e[v[r1].e[i]].r2);
			}
		}
	}
	else if(v[r2].e.size()==3)
	{
		for(int i=0;i<3;++i)
		{
			if(v[r2].e[i]!=mergepath.back())
			{
				q.push_back(v[r2].e[i]);
				if(e[v[r2].e[i]].r1!=r2)p.push_back(e[v[r2].e[i]].r1);
				else p.push_back(e[v[r2].e[i]].r2);
			}
		}
	}
	else//4
	{
		for(int i=0;i<4;++i)
		{
			if(v[r1].e[i]!=mergepath.front())
			{
				q.push_back(v[r1].e[i]);
				if(e[v[r1].e[i]].r1!=r1)p.push_back(e[v[r1].e[i]].r1);
				else p.push_back(e[v[r1].e[i]].r2);
			}
		}

		//d
		if(p[0]==p[1])
		{
			p[1]=p[2];p.pop_back();
			q[1]=q[2];q.pop_back();
		}
		else if(p[0]==p[2]||p[1]==p[2])
		{
			p.pop_back();
			q.pop_back();
		}
		else
		{
			//error
			std::cout<<"error: could not find 4 linked vertex!"<<std::endl;
			return -1;
		}
	}



	//Ot2ɕ

	//܂^[~l[^
	intv term;
	for(int i=0;i<mergepath.size();++i)
	{
		term.push_back(e[mergepath[i]].r1);
		term.push_back(e[mergepath[i]].r2);
	}
	//炩ɓ{ǂ܂B

	intv fill1(e.size()),fill2(e.size());
	fill1[q[0]]=1;
	FillEdge(p[0],e,v,fill1,term);
	fill2[q[1]]=1;
	FillEdge(p[1],e,v,fill2,term);

	std::vector<LoopEdge> sube1,sube2;
	for(int i=0;i<e.size();++i)
	{
		if(fill1[i])
		{
			sube1.push_back(e[i]);
		}
		if(fill2[i])
		{
			sube2.push_back(e[i]);
		}
	}

	for(int i=0;i<mergepath.size();++i)
	{

		LoopEdge le=e[mergepath[i]];
		le.id=1;
		sube1.push_back(le);
		sube2.push_back(le);
	}

	//_XV
	std::vector<LoopVertex> lv1(v.size()),lv2(v.size());
	for(int i=0;i<sube1.size();++i)
	{
		lv1[sube1[i].r1].v=sube1[i].r1;
		lv1[sube1[i].r1].e.push_back(i);
		lv1[sube1[i].r2].v=sube1[i].r2;
		lv1[sube1[i].r2].e.push_back(i);
	}
	for(int i=0;i<sube2.size();++i)
	{
		lv2[sube2[i].r1].v=sube2[i].r1;
		lv2[sube2[i].r1].e.push_back(i);
		lv2[sube2[i].r2].v=sube2[i].r2;
		lv2[sube2[i].r2].e.push_back(i);
	}

	//肩
	ExtractLoopSub(sube1,lv1,fout,f);
	ExtractLoopSub(sube2,lv2,fout,f);

	return 1;
}

int TekitohPhase::StretchEdge(int presentvertex,std::vector<LoopEdge> &e,std::vector<LoopVertex> &v,intv &checkededge,intv &path)
{
	//
	for(int i=0;i<v[presentvertex].e.size();++i)
	{
		if(e[v[presentvertex].e[i]].id==1)//ЕӔ!
		{
			path.push_back(v[presentvertex].e[i]);
			return 1;
		}
	}

	for(int i=0;i<v[presentvertex].e.size();++i)
	{
		int k=v[presentvertex].e[i];
		if(checkededge[k])continue;
		if(e[k].r1!=presentvertex)
		{
			checkededge[k]=1;
			int r=StretchEdge(e[k].r1,e,v,checkededge,path);
			if(r)
			{
				path.push_back(k);
				return 1;
			}
		}
		else if(e[k].r2!=presentvertex)
		{
			checkededge[k]=1;
			int r=StretchEdge(e[k].r2,e,v,checkededge,path);
			if(r)
			{
				path.push_back(k);
				return 1;
			}
		}
	}
	return 0;
}

int TekitohPhase::FillEdge(int presentvertex,std::vector<LoopEdge> &e,std::vector<LoopVertex> &v,intv &checkededge,intv &terminator)
{
	for(int i=0;i<terminator.size();++i)
	{
		if(presentvertex==terminator[i])return 1;
	}

	for(int i=0;i<v[presentvertex].e.size();++i)
	{
		int k=v[presentvertex].e[i];
		if(checkededge[k])continue;
		if(e[k].r1!=presentvertex)
		{
			checkededge[k]=1;
			FillEdge(e[k].r1,e,v,checkededge,terminator);
		}
		else if(e[k].r2!=presentvertex)
		{
			checkededge[k]=1;
			FillEdge(e[k].r2,e,v,checkededge,terminator);
		}
	}
	return 0;
}

int TekitohPhase::ReorderLoopEdge(std::vector<LoopEdge> &e,std::vector<LoopEdge> &out,intv &brink)
{
	int n=0;
	int inv=0;
	intv used(e.size());

	while(1)
	{
		LoopEdge ee=e[n];
		used[n]=1;
		ee.id=inv;
		if(inv)
		{
			std::swap(ee.r1,ee.r2);
		}
		if(e[n].id==1)brink.push_back(e[n].e);
		out.push_back(ee);
		if(out.size()==e.size())
		{
			return 0;
		}

		int found=-1;
		for(int i=0;i<e.size();++i)
		{
			if(used[i])continue;

			if(ee.r2==e[i].r1)
			{
				inv=0;
				found=n=i;
				break;
			}
			if(ee.r2==e[i].r2)
			{
				inv=1;
				found=n=i;
				break;
			}
		}
		if(found==-1)return -1;

	}
}



int TekitohPhase::MakePolygon()
{
	recpolygons.clear();

	int sz=mid.size();

	//loop to polygon
	//[v|S𐶐
//		for(int n=0;n<loop.size();++n)
	for(int n=0;n<loopface.size();++n)
	{
		//[vf[^_𐶐
		Vector2dv p;//_Ɋi[
		//int fc=loopface[n];
		int fc=loopface[n].v;

		intv usedbrinkedge;

		std::vector< LoopEdge > loopedge;
		loopedge.clear();
		for(int i=0;i<loopface[n].e.size();++i)
		{
			int ne=loopface[n].e[i];
			loopedge.push_back(LoopEdge(splitededges[fc][ne].r1,splitededges[fc][ne].r2,ne,splitededges[fc][ne].id));
		}

		std::vector< LoopEdge > outedge;
		ReorderLoopEdge(loopedge,outedge,usedbrinkedge);



		//for(int i=0;i<loop[n].size();++i)
		for(int i=0;i<outedge.size();++i)
		{
			/*int a=loop[n][i]/sz;
			int b=loop[n][i]%sz;
			//find
			int found=-1;
			int direction=1;
			for(int k=0;k<splitededges[fc].size();++k)
			{
				if(edgeused[k])continue;
				if(splitededges[fc][k].r1==a&&splitededges[fc][k].r2==b)
				{
					found=k;
					direction=1;
					break;
				}
				if(splitededges[fc][k].r2==a&&splitededges[fc][k].r1==b)
				{
					found=k;
					direction=-1;
					break;
				}
			}*/

			int found=outedge[i].e;
			int inv=outedge[i].id;

			if(found<0)continue;

			Edge &e=splitededges[fc][found];

			if(inv==0)
			{
				for(int j=0;j<e.points.size();++j)
				{
					if(p.empty())
					{
						p.push_back(e.points[j]);
					}
					else if(p.back()!=e.points[j])//d
					{
						p.push_back(e.points[j]);
					}
				}
			}
			else
			{
				for(int j=e.points.size()-1;j>=0;--j)
				{
					if(p.empty())
						p.push_back(e.points[j]);
					else if(p.back()!=e.points[j])
						p.push_back(e.points[j]);
				}
			}
		}

		if(p.back()==p.front())//[̏d
		{
			p.pop_back();
		}

		if(p.empty())
		{
			recpolygons.push_back(RecPolygon(fc,TIv(),EIv(),p,usedbrinkedge));
			continue;
		}

		//_E

		double sum=0;
		for(int i=0;i<p.size();++i)
		{
			sum+=p[i].ExProd(p[(i+1)%p.size()]);
		}
		if(sum>0)
		{
			//t
			Vector2dv q;
			for(int i=p.size()-1;i>=0;--i)
			{
				q.push_back(p[i]);
			}
			p=q;
		}

		//Op`
		EIv crs;
		TIv tri;
		for(int i=0;i<p.size();++i)
		{
			crs.push_back(EdgeIndex(i,(i+1)%p.size()));
		}
		DelauneyTriangulator<double>(p,crs,tri);

		//ӂ̕ӂƂ̂
		for(int i=0;i<crs.size();++i)
		{
			if(CheckOnEdge(p[crs[i].p1],p[crs[i].p2]))
			{
				//remove
				crs[i]=crs.back();
				crs.pop_back();
				--i;
			}
		}

		recpolygons.push_back(RecPolygon(fc,tri,crs,p,usedbrinkedge));

		std::cout<<p<<std::endl;
	}//end of making polygon


	//[GbWŗאڂ|SN
	for(int i=0;i+1<recpolygons.size();++i)
	{
		for(int j=0;j<recpolygons[i].brinkedges.size();++j)
		{
			int lk=recpolygons[i].brinkedges[j];
			int f0=recpolygons[i].face;

			int bk=splitededges[f0][lk].backlink;
			int f=bk%4;
			int e=bk/4;

			for(int k=i;k<recpolygons.size();++k)
			{
				if(recpolygons[k].face!=f)continue;

				for(int n=0;n<recpolygons[k].brinkedges.size();++n)
				{
					if(recpolygons[k].brinkedges[n]==e)
					{
						recpolygons[i].linkpolygon[j]=k;
						recpolygons[k].linkpolygon[n]=i;
					}
				}
			}
		}
	}//end of linking polygon

	//dbgout linkage
	std::cout<<"linkage"<<std::endl;
	for(int i=0;i<recpolygons.size();++i)
	{
		std::cout<<i<<":"<<recpolygons[i].linkpolygon<<std::endl;
	}

	//alert:܂Ă΃N͂ЂƂȂB

	return 1;
}

int TekitohPhase::OffsetPolygon()
{
	tts.clear();
	tts.resize(recpolygons.size());

	region=Rect2i(0,0,0,0);

	//GbW0,2,3܂ItZbg
	intv checked(recpolygons.size());
	intv target;
	target.push_back(0);
	for(int i=0;i<recpolygons.size();++i)
	{
		recpolygons[i].offset=Vector2i(0,0);
	}
	tts[0].SetFace(recpolygons[0].face,recpolygons[0].face);
	while(!target.empty())
	{
		int t=target.back();
		target.pop_back();
		checked[t]=1;

		for(int i=0;i<recpolygons[t].linkpolygon.size();++i)
		{
			int lp=recpolygons[t].linkpolygon[i];
			if(checked[lp])continue;

			Vector2i &tofs=recpolygons[t].offset;
			Vector2i &lofs=recpolygons[lp].offset;

			lofs=tofs;

			tts[t].child.push_back(lp);
			tts[lp].SetFace(recpolygons[t].face,recpolygons[lp].face);

			target.push_back(lp);
			int e=CheckFaceCrossID(recpolygons[t].face,recpolygons[lp].face);
			if(e==0)
			{
				if(recpolygons[t].face==0)
				{
					if(tofs.y&1) lofs.x++;
					else lofs.x--;
				}
				else//tf==3
				{
					if(tofs.y&1) lofs.x--;
					else lofs.x++;
				}
			}
			if(e==2)
			{
				//if(recpolygons[t].face==0)
				{
					if(tofs.y&1) lofs.y++;
					else lofs.y--;
				}
				/*else//tf==2
				{
					if(tofs.y&1) lofs.y++;
					else lofs.y--;
				}*/
			}
			if(e==3)
			{
				//if(recpolygons[t].face==1)
				{
					if(tofs.y&1) lofs.y--;
					else lofs.y++;
				}
				/*else//tf==3
				{
					if(tofs.y&1) lofs.y++;
					else lofs.y--;
				}*/
			}
		}	
	}

	//dbgout offset
	std::cout<<"offset"<<std::endl;
	for(int i=0;i<recpolygons.size();++i)
	{
		std::cout<<i<<":"<<recpolygons[i].offset<<std::endl;
		region.Insert(recpolygons[i].offset);
	}


	//dbgoutregion
	std::cout<<"region:"<<region<<std::endl;
	std::cout<<"diag:"<<region.Diagonal()<<std::endl;

	return 1;
}
	


