
#include <stdlib.h>
#include <stdio.h>
#include <iostream>
#include <math.h>
#include <matrix.h>
#include <matrixutil.h>

#include <ryaux.h>

#include "delauney.h"

using namespace Rydot;

typedef std::vector<Vector2d> Vector2dv;

//  2      3      2
//  +------+------+
//  |_    |_    |
//  |  _  |  _  |
//  |    _|    _|
//  +------+------+
//  0      1      0
int CheckVertex(const Vector2d &a,float rad);

//  +------+------+
//  |_    |_    |
//  | 0_1 | 2_3 |
//  |    _|    _|
//  +------+------+
// a is in [0:2][0:1]
int CheckFace(const Vector2d &a);

//ӏɂafc̒ɓ悤ɂ
//a[0:2][0:1]łƂB
Vector2d CorrectInFace(Vector2d a,int fc);


//     3      3
//  +------+------+
//  |_    |_    |
// 0| 4_ 1| 5_  |0
//  |    _|    _|
//  +------+------+
//     2      2

int CheckFaceCrossID(int af,int bf);


// not crossed: -1
// c,d͌_0,2,3
//  aɋ߂قc,
//  bɋ߂قd
int CheckFaceCross(const Vector2d &a,const Vector2d &b,Vector2d &c,Vector2d &d);

//a-bӏɂ̂Ă邩
int CheckOnEdge(const Vector2d &a,const Vector2d &b);

///////////////////////////////////////////////////////////
// class PartialEdge
//
// GbW

class PartialEdge
{
public:
	int a,b;
	int face;
	
	PartialEdge(){}
	PartialEdge(int _a,int _b,int _fc):a(_a),b(_b),face(_fc){}
};

typedef std::vector<PartialEdge> pev;

///////////////////////////////////////////////////////////
// class Edge 
//
// ؂ڃNX

class Edge
{
public:
	Vector2dv points;
	pev pedges;
	int crossability;
	int id;
	int backlink;
	int r1,r2;

	Edge(){}

	int CheckSelfEdgeCross(const Vector2d &a,const Vector2d &b);

	int Clear()
	{
		points.clear();
		pedges.clear();

		return 1;
	}

	bool Empty(){return points.empty();}

	int CheckCross(const Vector2d &a,const Vector2d &b,Vector2d &c);

	int Append(const Vector2d &a,int fc);

	Vector2d GetPresent()
	{
		return points.back();
	}
	
	Vector2d GetStart()
	{
		return points.front();
	}

	// se:split edge
	// c:split point
	std::vector<Edge> Split(int se,const Vector2d &sp);

	int Start(const Vector2d &a)
	{
		Clear();
		points.push_back(a);
		return 1;
	}
	
	int Validate(int _r1,int _r2)
	{
		r1=_r1;
		r2=_r2;

		return 1;
	}

	void SplitByFace(std::vector< std::vector<Edge> > &r,std::vector< std::pair<Vector2d,int> > &middlevertex);

};

typedef std::vector<Edge> ev;

///////////////////////////////////////////////////////////
// class RecPolygon
//
// fobOpL^NX

class RecPolygon
{
public:
	int face;
	TIv tris;
	EIv edges;
	Vector2dv p;
	intv brinkedges;//gp[GbW
	intv linkpolygon;//
	Vector2i offset;
	RecPolygon(int _f,const TIv &_tr,const EIv &_ed,const Vector2dv &_p,const intv &_be):face(_f),tris(_tr),edges(_ed),p(_p),brinkedges(_be){linkpolygon.resize(_be.size());}
	RecPolygon(){}
};



//WJAj邽߂̕ϊp̖
//Op`̒Sl
//eW
//->rotate_alpha(eW炻̎Op`ɂ)
//->offset_radius(Op`̓ډ~̔a+xɕsړ)
//->rotate_phi(ʊp)
//->offset_radius(Op`̓ډ~̔a+xɕsړ)
//->rotate_beta(ŎqWnɂȂ)
//->offset_center(phaseWOp`̌_)
//->shear(phaseWOp`)
//->offset_phase(phaseW_Ɉړ)
//΂͂^^;
class TransformTree
{
public:
	double rotate_alpha;
	double offset_radius;
	double rotate_beta;
	Vector2d offset_center;
	Matrix22d shear;
	Vector2d offset_phase;

	intv child;

public:
	//ʂʂւ킽鎞̍Wϊ
	void SetFace(int af,int bf);

};

//[vp
class LoopEdge
{
public:
	int r1,r2;
	int e;
	int id;
	LoopEdge(){}
	LoopEdge(int _r1,int _r2,int _e,int _id):r1(_r1),r2(_r2),e(_e),id(_id){}
};
class LoopVertex
{
public:
	int v;
	intv e;
	LoopVertex(){}
};

class TekitohPhase
{
public:
	ev validedges;
	Edge presentedge;

	int vertexused[6];
	int externalcrossnumber;
	Vector2d crossed[2];

	std::vector<LoopVertex> loopface;

	//std::vector< std::vector<int> > loop;
	//std::vector<int> loopface;
	std::vector< std::pair<Vector2d,int> > mid;
	std::vector< std::vector<Edge> > splitededges;//GbWʂƂɕ

	std::vector< RecPolygon > recpolygons;
	std::vector< TransformTree > tts;

	Rect2i region;

public:
	
	TekitohPhase()
	{
		Clear();
	}

	int SaveStroke(std::string fn);

	int LoadStroke(std::string fn);

	int SaveDelauney(std::string fn,int n);

	//̒_͂߂邩ǂ
	int Startable(const Vector2d &a);

	//̒_ŏI邩ǂ
	bool EndableVertex(int v);

	//̐؂ڂŏI邩ǂ
	bool EndableEdge(int e);

	//
	void Clear();

	//_̎gpJEg
	int CalcVertexUsed();

	//ǂ
	int CheckFinish();

	//_svdest؂ڂłȂĂ邩ǂ
	int CheckLink(int sv,int dest);

	//validedges[ne]ɂȂĂvalidedgeŝȂ
	//svɂȂĂ̂1([v)
	int CheckLoop(int ne,int sv);
	
	//؂ڃJbgɂȂ邩ǂ
	int CheckValidEdgeCross(const Vector2d &a,const Vector2d &b,Vector2d &c,int &ne);

	//؂ڂǉ
	//returns
	//0:cannot append
	//1:append succeed
	//2:crossed
	int Append(const Vector2d &a);

	//JbgKp(ꍇɂĂ͐؂ڃJbgƂȂ邱ƂB)
	int Apply();

	//JbgKp
	//ne:crossed valid edge
	//ce:crossed partialedge
	//c:cross point
	int ApplyCross(int ne,int ce,const Vector2d &c,int f);

	//
	int Failure()
	{
		presentedge.Clear();
		return 1;
	}

	//
	//܂ӖC
	int SplitByFace();


	//[vo
	int ExtractLoop(int f);

	int ExtractLoopSub(std::vector<LoopEdge> &e,std::vector<LoopVertex> &v,std::vector<LoopVertex> &fout,int f);

	//Еӂ̂钸_܂ŐL΂
	int StretchEdge(int presentvertex,std::vector<LoopEdge> &e,std::vector<LoopVertex> &v,intv &checkededge,intv &path);

	//OtʂԂ
	int FillEdge(int presentvertex,std::vector<LoopEdge> &e,std::vector<LoopVertex> &v,intv &checkededge,intv &terminator);

	//[v\ӂԂɕׂ
	int ReorderLoopEdge(std::vector<LoopEdge> &e,std::vector<LoopEdge> &out,intv &brink);


	//|S
	int MakePolygon();

	//|S̃N֌W
	int OffsetPolygon();

};
